@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * DisplayRenderer.cc
- * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -17,13 +17,14 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "DjVuDocument.hh"
 #include "DisplayRenderer.hh"
 #include "Utils.hh"
 
 #include <poppler-document.h>
 #include <poppler-page.h>
 
-void DisplayRenderer::adjustImage(const Cairo::RefPtr<Cairo::ImageSurface> &surf, int brightness, int contrast, bool invert) const {
+void DisplayRenderer::adjustImage(const Cairo::RefPtr<Cairo::ImageSurface>& surf, int brightness, int contrast, bool invert) const {
 	if(brightness == 0 && contrast == 0 && !invert) {
 		return;
 	}
@@ -39,9 +40,9 @@ void DisplayRenderer::adjustImage(const Cairo::RefPtr<Cairo::ImageSurface> &surf
 	uint8_t* data = surf->get_data();
 	#pragma omp parallel for schedule(static)
 	for(int i = 0; i < n; ++i) {
-		uint8_t& r = data[4*i + 2];
-		uint8_t& g = data[4*i + 1];
-		uint8_t& b = data[4*i + 0];
+		uint8_t& r = data[4 * i + 2];
+		uint8_t& g = data[4 * i + 1];
+		uint8_t& b = data[4 * i + 0];
 		// Brightness
 		r = dBr * (1.f - kBr) + r * kBr;
 		g = dBr * (1.f - kBr) + g * kBr;
@@ -92,13 +93,14 @@ Cairo::RefPtr<Cairo::ImageSurface> ImageRenderer::render(int /*page*/, double re
 	return surf;
 }
 
-PDFRenderer::PDFRenderer(const std::string& filename) : DisplayRenderer(filename) {
-	m_document = poppler_document_new_from_file(Glib::filename_to_uri(m_filename).c_str(), 0, 0);
+PDFRenderer::PDFRenderer(const std::string& filename, const Glib::ustring& password) : DisplayRenderer(filename) {
+	m_document = poppler_document_new_from_file(Glib::filename_to_uri(m_filename).c_str(), password.c_str(), 0);
 }
 
 PDFRenderer::~PDFRenderer() {
-	if(m_document)
+	if(m_document) {
 		g_object_unref(m_document);
+	}
 }
 
 Cairo::RefPtr<Cairo::ImageSurface> PDFRenderer::render(int page, double resolution) const {
@@ -132,3 +134,21 @@ Cairo::RefPtr<Cairo::ImageSurface> PDFRenderer::render(int page, double resoluti
 int PDFRenderer::getNPages() const {
 	return m_document ? poppler_document_get_n_pages(m_document) : 1;
 }
+
+
+DJVURenderer::DJVURenderer(const std::string& filename) : DisplayRenderer(filename) {
+	m_djvu = new DjVuDocument();
+	m_djvu->openFile(filename);
+}
+
+DJVURenderer::~DJVURenderer() {
+	delete m_djvu;
+}
+
+Cairo::RefPtr<Cairo::ImageSurface> DJVURenderer::render(int page, double resolution) const {
+	return m_djvu->image(page, resolution);
+}
+
+int DJVURenderer::getNPages() const {
+	return m_djvu->pageCount();
+}
