@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Recognizer.cc
- * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -17,21 +17,22 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "MainWindow.hh"
-#include "Config.hh"
+#include "ConfigSettings.hh"
 #include "Displayer.hh"
+#include "MainWindow.hh"
 #include "OutputEditor.hh"
 #include "Recognizer.hh"
-#include "TessdataManager.hh"
 #include "Utils.hh"
 
 #include <gtkspellmm.h>
 #include <csignal>
 #include <cstring>
+#define USE_STD_NAMESPACE
 #include <tesseract/baseapi.h>
 #include <tesseract/strngs.h>
 #include <tesseract/genericvector.h>
 #include <tesseract/ocrclass.h>
+#undef USE_STD_NAMESPACE
 #include <unistd.h>
 #include <setjmp.h>
 
@@ -41,26 +42,23 @@
 #endif
 
 
-struct Recognizer::ProgressMonitor : public MainWindow::ProgressMonitor {
+class Recognizer::ProgressMonitor : public MainWindow::ProgressMonitor {
+public:
 	ETEXT_DESC desc;
-	bool canceled = false;
-	int donePages = 0;
-	int nPages;
 
-	ProgressMonitor(int _nPages) {
+	ProgressMonitor(int nPages) : MainWindow::ProgressMonitor(nPages) {
 		desc.progress = 0;
 		desc.cancel = cancelCallback;
 		desc.cancel_this = this;
-		nPages = _nPages;
 	}
-	int getProgress() {
-		return 100 * ((donePages + desc.progress / 100.) / nPages);
-	}
-	void cancel() {
-		canceled = true;
+	int getProgress() const override {
+		Glib::Threads::Mutex::Lock lock(m_mutex);
+		return 100.0 * ((m_progress + desc.progress / 100.0) / m_total);
 	}
 	static bool cancelCallback(void* instance, int /*words*/) {
-		return reinterpret_cast<ProgressMonitor*>(instance)->canceled;
+		ProgressMonitor* monitor = reinterpret_cast<ProgressMonitor*>(instance);
+		Glib::Threads::Mutex::Lock lock(monitor->m_mutex);
+		return monitor->m_cancelled;
 	}
 };
 
@@ -94,47 +92,45 @@ protected:
 	}
 
 private:
+	ClassData m_classdata;
 	bool m_selected = false;
 	bool m_ignoreNextActivate = false;
 };
 
-Recognizer::Recognizer() {
-	m_menuPages = MAIN->getWidget("menu:recognize.pages");
-	m_menuLanguages = MAIN->getWidget("menu:main.languages");
-	m_pagesDialog = MAIN->getWidget("dialog:recognize.pages");
-	m_pagesEntry = MAIN->getWidget("entry:dialog.pages");
-	m_langLabel = MAIN->getWidget("label:main.recognize.lang");
-	m_modeLabel = MAIN->getWidget("label:main.recognize.mode");
-	m_pageAreaLabel = MAIN->getWidget("label:dialog.regions");
-	m_pageAreaCombo = MAIN->getWidget("comboboxtext:dialog.regions");
-	Gtk::MenuButton* recognizeBtn = MAIN->getWidget("menubutton:main.languages");
-	recognizeBtn->set_menu(*m_menuLanguages);
-	m_recognizeBtn = MAIN->getWidget("button:main.recognize");
-
-	CONNECT(m_recognizeBtn, clicked, [this] { recognizeButtonClicked(); });
-	CONNECT(MAIN->getWidget("menuitem:recognize.pages.current").as<Gtk::MenuItem>(), activate, [this] { recognizeCurrentPage(); });
-	CONNECT(MAIN->getWidget("menuitem:recognize.pages.multiple").as<Gtk::MenuItem>(), activate, [this] { recognizeMultiplePages();; });
-	CONNECTS(m_pagesEntry, focus_in_event, [](GdkEventFocus*, Gtk::Entry* e) {
-		Utils::clear_error_state(e);
+Recognizer::Recognizer(const Ui::MainWindow& _ui)
+	: ui(_ui) {
+	CONNECT(ui.buttonRecognize, clicked, [this] { recognizeButtonClicked(); });
+	CONNECT(ui.menuitemRecognizeCurrent, activate, [this] { recognizeCurrentPage(); });
+	CONNECT(ui.menuitemRecognizeMultiple, activate, [this] { recognizeMultiplePages();; });
+	CONNECT(ui.entryPageRange, focus_in_event, [this](GdkEventFocus*) {
+		Utils::clear_error_state(ui.entryPageRange);
 		return false;
 	});
 
-	MAIN->getConfig()->addSetting(new VarSetting<Glib::ustring>("language"));
-	MAIN->getConfig()->addSetting(new ComboSetting("ocrregionstrategy", MAIN->getWidget("comboboxtext:dialog.regions")));
-	MAIN->getConfig()->addSetting(new SwitchSettingT<Gtk::CheckButton>("ocraddsourcefilename", MAIN->getWidget("checkbutton:dialog.pages.prepend.filename")));
-	MAIN->getConfig()->addSetting(new SwitchSettingT<Gtk::CheckButton>("ocraddsourcepage", MAIN->getWidget("checkbutton:dialog.pages.prepend.page")));
-	MAIN->getConfig()->addSetting(new VarSetting<int>("psm"));
+	ADD_SETTING(VarSetting<Glib::ustring>("language"));
+	ADD_SETTING(ComboSetting("ocrregionstrategy", ui.comboPageRangeRegions));
+	ADD_SETTING(SwitchSettingT<Gtk::CheckButton>("ocraddsourcefilename", ui.checkPageRangePrependFile));
+	ADD_SETTING(SwitchSettingT<Gtk::CheckButton>("ocraddsourcepage", ui.checkPageRangePrependPage));
+	ADD_SETTING(VarSetting<int>("psm"));
 }
 
 std::vector<Glib::ustring> Recognizer::getAvailableLanguages() const {
-	tesseract::TessBaseAPI tess;
-	initTesseract(tess);
+	tesseract::TessBaseAPI tess = initTesseract();
 	GenericVector<STRING> availLanguages;
 	tess.GetAvailableLanguagesAsVector(&availLanguages);
 	std::vector<Glib::ustring> result;
 	for(int i = 0; i < availLanguages.size(); ++i) {
 		result.push_back(availLanguages[i].string());
 	}
+	std::sort(result.begin(), result.end(), [](const Glib::ustring & s1, const Glib::ustring & s2) {
+		bool s1Script = s1.substr(0, 6) == "script" || s1.substr(0, 1) == s1.substr(0, 1).uppercase();
+		bool s2Script = s2.substr(0, 6) == "script" || s2.substr(0, 1) == s2.substr(0, 1).uppercase();
+		if(s1Script != s2Script) {
+			return !s1Script;
+		} else {
+			return s1 < s2;
+		}
+	});
 	return result;
 }
 
@@ -147,11 +143,11 @@ static void tessCrashHandler(int /*signal*/) {
 	int bytesRead = 0;
 	Glib::ustring captured;
 	do {
-		if((bytesRead = read(g_pipe[0], buf, sizeof(buf)-1)) > 0) {
+		if((bytesRead = read(g_pipe[0], buf, sizeof(buf) - 1)) > 0) {
 			buf[bytesRead] = 0;
 			captured += buf;
 		}
-	} while(bytesRead == sizeof(buf)-1);
+	} while(bytesRead == sizeof(buf) - 1);
 	tesseract::TessBaseAPI tess;
 	Glib::ustring errMsg = Glib::ustring::compose(_("Tesseract crashed with the following message:\n\n"
 	                       "%1\n\n"
@@ -164,9 +160,10 @@ static void tessCrashHandler(int /*signal*/) {
 	longjmp(g_restore_point, SIGSEGV);
 }
 
-bool Recognizer::initTesseract(tesseract::TessBaseAPI& tess, const char* language) const {
-	std::string current = setlocale(LC_NUMERIC, NULL);
-	setlocale(LC_NUMERIC, "C");
+tesseract::TessBaseAPI Recognizer::initTesseract(const char* language, bool* ok) const {
+	std::string current = setlocale(LC_ALL, NULL);
+	setlocale(LC_ALL, "C");
+	tesseract::TessBaseAPI tess;
 	// unfortunately tesseract creates deliberate segfaults when an error occurs
 	std::signal(SIGSEGV, tessCrashHandler);
 	pipe(g_pipe);
@@ -182,16 +179,19 @@ bool Recognizer::initTesseract(tesseract::TessBaseAPI& tess, const char* languag
 	}
 	dup2(oldstderr, fileno(stderr));
 	std::signal(SIGSEGV, MainWindow::signalHandler);
-	setlocale(LC_NUMERIC, current.c_str());
+	setlocale(LC_ALL, current.c_str());
 
 	close(g_pipe[0]);
 	close(g_pipe[1]);
-	return ret != -1;
+	if(ok) {
+		*ok = ret != -1;
+	}
+	return tess;
 }
 
 void Recognizer::updateLanguagesMenu() {
-	m_menuLanguages->foreach([this](Gtk::Widget& w) {
-	m_menuLanguages->remove(w);
+	ui.menuLanguages->foreach([this](Gtk::Widget& w) {
+	ui.menuLanguages->remove(w);
 	});
 	m_langMenuRadioGroup = Gtk::RadioButtonGroup();
 	m_langMenuCheckGroup = std::vector<std::pair<Gtk::CheckMenuItem*, Glib::ustring>>();
@@ -201,7 +201,7 @@ void Recognizer::updateLanguagesMenu() {
 	Gtk::RadioMenuItem* activeitem = nullptr;
 	bool haveOsd = false;
 
-	std::vector<Glib::ustring> parts = Utils::string_split(MAIN->getConfig()->getSetting<VarSetting<Glib::ustring>>("language")->getValue(), ':');
+	std::vector<Glib::ustring> parts = Utils::string_split(ConfigSettings::get<VarSetting<Glib::ustring>>("language")->getValue(), ':');
 	Config::Lang curlang = {parts.empty() ? "eng" : parts[0], parts.size() < 2 ? "" : parts[1]};
 
 	std::vector<Glib::ustring> dicts = GtkSpell::Checker::get_language_list();
@@ -210,7 +210,7 @@ void Recognizer::updateLanguagesMenu() {
 
 	if(availLanguages.empty()) {
 		Utils::message_dialog(Gtk::MESSAGE_ERROR, _("No languages available"), _("No tesseract languages are available for use. Recognition will not work."));
-		m_langLabel->set_text("");
+		ui.labelRecognizeLang->set_text("");
 	}
 
 	// Add menu items for languages, with spelling submenu if available
@@ -248,7 +248,7 @@ void Recognizer::updateLanguagesMenu() {
 				submenu->append(*curitem);
 			}
 			item->set_submenu(*submenu);
-			m_menuLanguages->append(*item);
+			ui.menuLanguages->append(*item);
 		} else {
 			curitem = Gtk::manage(new Gtk::RadioMenuItem(m_langMenuRadioGroup, lang.name));
 			CONNECT(curitem, toggled, [this, curitem, lang] { setLanguage(curitem, lang); });
@@ -256,13 +256,13 @@ void Recognizer::updateLanguagesMenu() {
 				curlang = lang;
 				activeitem = curitem;
 			}
-			m_menuLanguages->append(*curitem);
+			ui.menuLanguages->append(*curitem);
 		}
 	}
 	// Add multilanguage menu
 	bool isMultilingual = false;
 	if(!availLanguages.empty()) {
-		m_menuLanguages->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
+		ui.menuLanguages->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
 		m_multilingualRadio = Gtk::manage(new MultilingualMenuItem(m_langMenuRadioGroup, _("Multilingual")));
 		Gtk::Menu* submenu = Gtk::manage(new Gtk::Menu);
 		isMultilingual = curlang.prefix.find('+') != curlang.prefix.npos;
@@ -277,10 +277,10 @@ void Recognizer::updateLanguagesMenu() {
 			}
 			Gtk::CheckMenuItem* item = Gtk::manage(new Gtk::CheckMenuItem(lang.name));
 			item->set_active(isMultilingual && std::find(sellangs.begin(), sellangs.end(), lang.prefix) != sellangs.end());
-			CONNECT(item, button_press_event, [this,item](GdkEventButton* ev) {
+			CONNECT(item, button_press_event, [this, item](GdkEventButton * ev) {
 				return onMultilingualItemButtonEvent(ev, item);
 			}, false);
-			CONNECT(item, button_release_event, [this,item](GdkEventButton* ev) {
+			CONNECT(item, button_release_event, [this, item](GdkEventButton * ev) {
 				return onMultilingualItemButtonEvent(ev, item);
 			}, false);
 			//		CONNECT(item, toggled, [this]{ setMultiLanguage(); });
@@ -289,13 +289,13 @@ void Recognizer::updateLanguagesMenu() {
 		}
 		m_multilingualRadio->set_submenu(*submenu);
 		CONNECT(m_multilingualRadio, toggled, [this] { if(m_multilingualRadio->get_active()) setMultiLanguage(); });
-		CONNECT(submenu, button_press_event, [this](GdkEventButton* ev) {
+		CONNECT(submenu, button_press_event, [this](GdkEventButton * ev) {
 			return onMultilingualMenuButtonEvent(ev);
 		}, false);
-		CONNECT(submenu, button_release_event, [this](GdkEventButton* ev) {
+		CONNECT(submenu, button_release_event, [this](GdkEventButton * ev) {
 			return onMultilingualMenuButtonEvent(ev);
 		}, false);
-		m_menuLanguages->append(*m_multilingualRadio);
+		ui.menuLanguages->append(*m_multilingualRadio);
 	}
 	if(isMultilingual) {
 		activeitem = m_multilingualRadio;
@@ -305,9 +305,9 @@ void Recognizer::updateLanguagesMenu() {
 	}
 
 	// Add PSM items
-	m_menuLanguages->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
+	ui.menuLanguages->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
 	Gtk::Menu* psmMenu = Gtk::manage(new Gtk::Menu);
-	m_currentPsmMode = MAIN->getConfig()->getSetting<VarSetting<int>>("psm")->getValue();
+	m_currentPsmMode = ConfigSettings::get<VarSetting<int>>("psm")->getValue();
 
 	struct PsmEntry {
 		Glib::ustring label;
@@ -324,7 +324,8 @@ void Recognizer::updateLanguagesMenu() {
 		PsmEntry{_("Assume a single word"), tesseract::PSM_SINGLE_WORD, false},
 		PsmEntry{_("Assume a single word in a circle"), tesseract::PSM_CIRCLE_WORD, false},
 		PsmEntry{_("Sparse text in no particular order"), tesseract::PSM_SPARSE_TEXT, false},
-		PsmEntry{_("Sparse text with orientation and script detection"), tesseract::PSM_SPARSE_TEXT_OSD, true}};
+		PsmEntry{_("Sparse text with orientation and script detection"), tesseract::PSM_SPARSE_TEXT_OSD, true}
+	};
 	for(const auto& entry : psmModes) {
 		Gtk::RadioMenuItem* item = Gtk::manage(new Gtk::RadioMenuItem(m_psmRadioGroup, entry.label));
 		item->set_sensitive(!entry.requireOsd || haveOsd);
@@ -332,7 +333,7 @@ void Recognizer::updateLanguagesMenu() {
 		CONNECT(item, toggled, [this, entry, item] {
 			if(item->get_active()) {
 				m_currentPsmMode = entry.psmMode;
-				MAIN->getConfig()->getSetting<VarSetting<int>>("psm")->setValue(entry.psmMode);
+				ConfigSettings::get<VarSetting<int>>("psm")->setValue(entry.psmMode);
 			}
 		});
 		psmMenu->append(*item);
@@ -340,30 +341,30 @@ void Recognizer::updateLanguagesMenu() {
 
 	Gtk::MenuItem* psmItem = Gtk::manage(new Gtk::MenuItem(_("Page segmentation mode")));
 	psmItem->set_submenu(*psmMenu);
-	m_menuLanguages->append(*psmItem);
+	ui.menuLanguages->append(*psmItem);
 
 	// Add installer item
-	m_menuLanguages->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
+	ui.menuLanguages->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
 	Gtk::MenuItem* manageItem = Gtk::manage(new Gtk::MenuItem(_("Manage languages...")));
-	CONNECT(manageItem, activate, [this] { manageInstalledLanguages(); });
-	m_menuLanguages->append(*manageItem);
+	CONNECT(manageItem, activate, [this] { MAIN->manageLanguages(); });
+	ui.menuLanguages->append(*manageItem);
 
-	m_menuLanguages->show_all();
+	ui.menuLanguages->show_all();
 	if(activeitem) {
 		activeitem->set_active(true);
 		activeitem->toggled(); // Ensure signal is emitted
 	}
 }
 
-void Recognizer::setLanguage(const Gtk::RadioMenuItem* item, const Config::Lang &lang) {
+void Recognizer::setLanguage(const Gtk::RadioMenuItem* item, const Config::Lang& lang) {
 	if(item->get_active()) {
 		if(!lang.code.empty()) {
-			m_langLabel->set_markup(Glib::ustring::compose("<small> %1 (%2)</small>", lang.name, lang.code));
+			ui.labelRecognizeLang->set_markup(Glib::ustring::compose("<small> %1 (%2)</small>", lang.name, lang.code));
 		} else {
-			m_langLabel->set_markup(Glib::ustring::compose("<small> %1</small>", lang.name));
+			ui.labelRecognizeLang->set_markup(Glib::ustring::compose("<small> %1</small>", lang.name));
 		}
 		m_curLang = lang;
-		MAIN->getConfig()->getSetting<VarSetting<Glib::ustring>>("language")->setValue(lang.prefix + ":" + lang.code);
+		ConfigSettings::get<VarSetting<Glib::ustring>>("language")->setValue(lang.prefix + ":" + lang.code);
 		m_signal_languageChanged.emit(m_curLang);
 	}
 }
@@ -380,57 +381,63 @@ void Recognizer::setMultiLanguage() {
 		langs = "eng+";
 	}
 	langs = langs.substr(0, langs.length() - 1);
-	m_langLabel->set_markup("<small>" + langs + "</small>");
+	ui.labelRecognizeLang->set_markup("<small>" + langs + "</small>");
 	m_curLang = {langs, "", "Multilingual"};
-	MAIN->getConfig()->getSetting<VarSetting<Glib::ustring>>("language")->setValue(langs + ":");
+	ConfigSettings::get<VarSetting<Glib::ustring>>("language")->setValue(langs + ":");
 	m_signal_languageChanged.emit(m_curLang);
 }
 
 void Recognizer::setRecognizeMode(const Glib::ustring& mode) {
-	m_modeLabel->set_markup(Glib::ustring::compose("<small>%1</small>", mode));
+	ui.labelRecognizeMode->set_markup(Glib::ustring::compose("<small>%1</small>", mode));
 }
 
 std::vector<int> Recognizer::selectPages(bool& autodetectLayout) {
 	int nPages = MAIN->getDisplayer()->getNPages();
 
-	m_pagesEntry->set_text(Glib::ustring::compose("1-%1", nPages));
-	m_pagesEntry->grab_focus();
-	m_pageAreaLabel->set_visible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
-	m_pageAreaCombo->set_visible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
-	MAIN->getWidget("box:dialog.pages.prepend")->set_visible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
-	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(m_pageAreaCombo->get_model());
-	int col = m_pageAreaCombo->get_entry_text_column();
+	ui.entryPageRange->set_text(Glib::ustring::compose("1-%1", nPages));
+	ui.entryPageRange->grab_focus();
+	ui.labelPageRangeRegions->set_visible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
+	ui.comboPageRangeRegions->set_visible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
+	ui.boxPageRangePrepend->set_visible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
+	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(ui.comboPageRangeRegions->get_model());
+	int col = ui.comboPageRangeRegions->get_entry_text_column();
 	store->children()[0]->set_value<Glib::ustring>(col, MAIN->getDisplayer()->hasMultipleOCRAreas() ? _("Current selection") : _("Entire page"));
 
+	static Glib::RefPtr<Glib::Regex> validateRegEx = Glib::Regex::create("^[\\d,\\-\\s]+$");
 	std::vector<int> pages;
-	if(m_pagesDialog->run() == Gtk::RESPONSE_OK) {
-		Glib::ustring text = m_pagesEntry->get_text();
-		text = Glib::Regex::create("\\s+")->replace(text, 0, "", static_cast<Glib::RegexMatchFlags>(0));
-		std::vector<Glib::ustring>&& blocks = Utils::string_split(text, ',', false);
-		for(const Glib::ustring& block : blocks) {
-			std::vector<Glib::ustring>&& ranges = Utils::string_split(block, '-', false);
-			if(ranges.size() == 1) {
-				int page = atoi(ranges[0].c_str());
-				if(page > 0 && page <= nPages) {
-					pages.push_back(page);
+	while(ui.dialogPageRange->run() == Gtk::RESPONSE_OK) {
+		pages.clear();
+		Glib::ustring text = ui.entryPageRange->get_text();
+		if(validateRegEx->match(text)) {
+			text = Glib::Regex::create("\\s+")->replace(text, 0, "", static_cast<Glib::RegexMatchFlags>(0));
+			std::vector<Glib::ustring> blocks = Utils::string_split(text, ',', false);
+			for(const Glib::ustring& block : blocks) {
+				std::vector<Glib::ustring> ranges = Utils::string_split(block, '-', false);
+				if(ranges.size() == 1) {
+					int page = Utils::parseInt(ranges[0]);
+					if(page > 0 && page <= nPages) {
+						pages.push_back(page);
+					}
+				} else if(ranges.size() == 2) {
+					int start = std::max(1, Utils::parseInt(ranges[0]));
+					int end = std::min(nPages, Utils::parseInt(ranges[1]));
+					for(int page = start; page <= end; ++page) {
+						pages.push_back(page);
+					}
+				} else {
+					pages.clear();
+					break;
 				}
-			} else if(ranges.size() == 2) {
-				int start = std::max(1, atoi(ranges[0].c_str()));
-				int end = std::min(nPages, atoi(ranges[1].c_str()));
-				for(int page = start; page <= end; ++page) {
-					pages.push_back(page);
-				}
-			} else {
-				pages.clear();
-				break;
 			}
 		}
 		if(pages.empty()) {
-			Utils::set_error_state(m_pagesEntry);
+			Utils::set_error_state(ui.entryPageRange);
+		} else {
+			break;
 		}
 	}
-	autodetectLayout = m_pageAreaCombo->get_visible() ? m_pageAreaCombo->get_active_row_number() == 1 : false;
-	m_pagesDialog->hide();
+	autodetectLayout = ui.comboPageRangeRegions->get_visible() ? ui.comboPageRangeRegions->get_active_row_number() == 1 : false;
+	ui.dialogPageRange->hide();
 	std::sort(pages.begin(), pages.end());
 	return pages;
 }
@@ -440,8 +447,8 @@ void Recognizer::recognizeButtonClicked() {
 	if(nPages == 1) {
 		recognize({MAIN->getDisplayer()->getCurrentPage()});
 	} else {
-		auto positioner = sigc::bind(sigc::ptr_fun(Utils::popup_positioner), m_recognizeBtn, m_menuPages, false, true);
-		m_menuPages->popup(positioner, 0, gtk_get_current_event_time());
+		auto positioner = sigc::bind(sigc::ptr_fun(Utils::popup_positioner), ui.buttonRecognize, ui.menuRecognizePages, false, true);
+		ui.menuRecognizePages->popup(positioner, 0, gtk_get_current_event_time());
 	}
 }
 
@@ -455,11 +462,12 @@ void Recognizer::recognizeMultiplePages() {
 	recognize(pages, autodetectLayout);
 }
 
-void Recognizer::recognize(const std::vector<int> &pages, bool autodetectLayout) {
-	tesseract::TessBaseAPI tess;
-	bool prependFile = pages.size() > 1 && MAIN->getConfig()->getSetting<SwitchSetting>("ocraddsourcefilename")->getValue();
-	bool prependPage = pages.size() > 1 && MAIN->getConfig()->getSetting<SwitchSetting>("ocraddsourcepage")->getValue();
-	if(initTesseract(tess, m_curLang.prefix.c_str())) {
+void Recognizer::recognize(const std::vector<int>& pages, bool autodetectLayout) {
+	bool prependFile = pages.size() > 1 && ConfigSettings::get<SwitchSetting>("ocraddsourcefilename")->getValue();
+	bool prependPage = pages.size() > 1 && ConfigSettings::get<SwitchSetting>("ocraddsourcepage")->getValue();
+	bool ok = false;
+	tesseract::TessBaseAPI tess = initTesseract(m_curLang.prefix.c_str(), &ok);
+	if(ok) {
 		Glib::ustring failed;
 		tess.SetPageSegMode(static_cast<tesseract::PageSegMode>(m_currentPsmMode));
 		OutputEditor::ReadSessionData* readSessionData = MAIN->getOutputEditor()->initRead(tess);
@@ -472,22 +480,23 @@ void Recognizer::recognize(const std::vector<int> &pages, bool autodetectLayout)
 			for(int page : pages) {
 				monitor.desc.progress = 0;
 				++idx;
-				Glib::signal_idle().connect_once([=] { MAIN->pushState(MainWindow::State::Busy, Glib::ustring::compose(_("Recognizing page %1 (%2 of %3)"), page, idx, npages)); });
+				Glib::signal_idle().connect_once([ = ] { MAIN->pushState(MainWindow::State::Busy, Glib::ustring::compose(_("Recognizing page %1 (%2 of %3)"), page, idx, npages)); });
 
-				bool success = false;
-				Utils::runInMainThreadBlocking([&] { success = setPage(page, autodetectLayout); });
-				if(!success) {
+				PageData pageData;
+				Utils::runInMainThreadBlocking([&] { pageData = setPage(page, autodetectLayout); });
+				if(!pageData.success) {
 					failed.append(Glib::ustring::compose(_("\n- Page %1: failed to render page"), page));
 					MAIN->getOutputEditor()->readError(_("\n[Failed to recognize page %1]\n"), readSessionData);
 					continue;
 				}
-				readSessionData->file = MAIN->getDisplayer()->getCurrentImage(readSessionData->page);
-				readSessionData->angle = MAIN->getDisplayer()->getCurrentAngle();
-				readSessionData->resolution = MAIN->getDisplayer()->getCurrentResolution();
+				readSessionData->file = pageData.filename;
+				readSessionData->page = pageData.page;
+				readSessionData->angle = pageData.angle;
+				readSessionData->resolution = pageData.resolution;
 				bool firstChunk = true;
 				bool newFile = readSessionData->file != prevFile;
 				prevFile = readSessionData->file;
-				for(const Cairo::RefPtr<Cairo::ImageSurface>& image : MAIN->getDisplayer()->getOCRAreas()) {
+				for(const Cairo::RefPtr<Cairo::ImageSurface>& image : pageData.ocrAreas) {
 					readSessionData->prependPage = prependPage && firstChunk;
 					readSessionData->prependFile = prependFile && (readSessionData->prependPage || newFile);
 					firstChunk = false;
@@ -495,14 +504,14 @@ void Recognizer::recognize(const std::vector<int> &pages, bool autodetectLayout)
 					tess.SetImage(image->get_data(), image->get_width(), image->get_height(), 4, image->get_stride());
 					tess.SetSourceResolution(MAIN->getDisplayer()->getCurrentResolution());
 					tess.Recognize(&monitor.desc);
-					if(!monitor.canceled) {
+					if(!monitor.cancelled()) {
 						MAIN->getOutputEditor()->read(tess, readSessionData);
 					}
 				}
 
 				Glib::signal_idle().connect_once([] { MAIN->popState(); });
-				++monitor.donePages;
-				if(monitor.canceled) {
+				monitor.increaseProgress();
+				if(monitor.cancelled()) {
 					break;
 				}
 			}
@@ -516,9 +525,10 @@ void Recognizer::recognize(const std::vector<int> &pages, bool autodetectLayout)
 	}
 }
 
-bool Recognizer::recognizeImage(const Cairo::RefPtr<Cairo::ImageSurface> &img, OutputDestination dest) {
-	tesseract::TessBaseAPI tess;
-	if(!initTesseract(tess, m_curLang.prefix.c_str())) {
+bool Recognizer::recognizeImage(const Cairo::RefPtr<Cairo::ImageSurface>& img, OutputDestination dest) {
+	bool ok = false;
+	tesseract::TessBaseAPI tess = initTesseract(m_curLang.prefix.c_str(), &ok);
+	if(!ok) {
 		Utils::message_dialog(Gtk::MESSAGE_ERROR, _("Recognition errors occurred"), _("Failed to initialize tesseract"));
 		return false;
 	}
@@ -532,7 +542,7 @@ bool Recognizer::recognizeImage(const Cairo::RefPtr<Cairo::ImageSurface> &img, O
 		readSessionData->resolution = MAIN->getDisplayer()->getCurrentResolution();
 		Utils::busyTask([&] {
 			tess.Recognize(&monitor.desc);
-			if(!monitor.canceled) {
+			if(!monitor.cancelled()) {
 				MAIN->getOutputEditor()->read(tess, readSessionData);
 			}
 			return true;
@@ -542,7 +552,7 @@ bool Recognizer::recognizeImage(const Cairo::RefPtr<Cairo::ImageSurface> &img, O
 		Glib::ustring output;
 		if(Utils::busyTask([&] {
 		tess.Recognize(&monitor.desc);
-			if(!monitor.canceled) {
+			if(!monitor.cancelled()) {
 				char* text = tess.GetUTF8Text();
 				output = text;
 				delete[] text;
@@ -557,15 +567,19 @@ bool Recognizer::recognizeImage(const Cairo::RefPtr<Cairo::ImageSurface> &img, O
 	return true;
 }
 
-bool Recognizer::setPage(int page, bool autodetectLayout) {
-	bool success = true;
-	if(page != MAIN->getDisplayer()->getCurrentPage()) {
-		success = MAIN->getDisplayer()->setCurrentPage(page);
+Recognizer::PageData Recognizer::setPage(int page, bool autodetectLayout) {
+	PageData pageData;
+	pageData.success = MAIN->getDisplayer()->setup(&page);
+	if(pageData.success) {
+		if(autodetectLayout) {
+			MAIN->getDisplayer()->autodetectOCRAreas();
+		}
+		pageData.filename = MAIN->getDisplayer()->getCurrentImage(pageData.page);
+		pageData.angle = MAIN->getDisplayer()->getCurrentAngle();
+		pageData.resolution = MAIN->getDisplayer()->getCurrentResolution();
+		pageData.ocrAreas = MAIN->getDisplayer()->getOCRAreas();
 	}
-	if(autodetectLayout) {
-		MAIN->getDisplayer()->autodetectOCRAreas();
-	}
-	return success;
+	return pageData;
 }
 
 bool Recognizer::onMultilingualMenuButtonEvent(GdkEventButton* ev) {
@@ -598,7 +612,3 @@ bool Recognizer::onMultilingualItemButtonEvent(GdkEventButton* ev, Gtk::CheckMen
 	}
 	return false;
 }
-
-void Recognizer::manageInstalledLanguages() {
-	TessdataManager::exec();
-}
