@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * SourceManager.cc
- * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -23,106 +23,169 @@
 #include "Utils.hh"
 
 #include <pangomm.h>
+#include <poppler-document.h>
 
-SourceManager::SourceManager() {
-	m_notebook = MAIN->getWidget("notebook:sources");
-	m_listView = MAIN->getWidget("treeview:input.images");
-	m_addButton = MAIN->getWidget("button:sources.images.add");
-	m_addButtonMenu = MAIN->getWidget("menubutton:sources.images.add");
-	m_removeButton = MAIN->getWidget("button:sources.images.remove");
-	m_deleteButton = MAIN->getWidget("button:sources.images.delete");
-	m_clearButton = MAIN->getWidget("button:sources.images.clear");
-	m_pasteItem = MAIN->getWidget("menuitem:sources.images.paste");
 
-	m_listView->set_model(Gtk::ListStore::create(m_listViewCols));
-	m_listView->append_column("", m_listViewCols.filename);
-	Gtk::TreeViewColumn* col = m_listView->get_column(0);
+SourceManager::SourceManager(const Ui::MainWindow& _ui)
+	: ui(_ui) {
+	ui.treeviewSources->set_model(Gtk::ListStore::create(m_listViewCols));
+	ui.treeviewSources->append_column("", m_listViewCols.filename);
+	Gtk::TreeViewColumn* col = ui.treeviewSources->get_column(0);
 	col->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
 	Gtk::CellRendererText* cell = static_cast<Gtk::CellRendererText*>(col->get_cells().front());
 	cell->property_ellipsize() = Pango::ELLIPSIZE_END;
-	m_listView->set_fixed_height_mode();
+	ui.treeviewSources->set_fixed_height_mode();
 
 	Glib::RefPtr<Gtk::RecentFilter> recentFilter = Gtk::RecentFilter::create();
 	recentFilter->add_pixbuf_formats();
 	recentFilter->add_mime_type("application/pdf");
 	recentFilter->add_pattern("*.pdf");
+	recentFilter->add_mime_type("image/vnd.djvu");
+	recentFilter->add_pattern("*.djvu");
 	Gtk::RecentChooserMenu* recentChooser = Gtk::manage(new Gtk::RecentChooserMenu());
 	recentChooser->set_filter(recentFilter);
 	recentChooser->set_local_only(false);
 	recentChooser->set_show_not_found(false);
 	recentChooser->set_show_tips(true);
 	recentChooser->set_sort_type(Gtk::RECENT_SORT_MRU);
-	MAIN->getWidget("menuitem:sources.images.recent").as<Gtk::MenuItem>()->set_submenu(*recentChooser);
+	ui.menuitemSourcesRecent->set_submenu(*recentChooser);
 
 	m_clipboard = Gtk::Clipboard::get_for_display(Gdk::Display::get_default());
 
-	Gtk::ToggleButton* toggleSourcesBtn = MAIN->getWidget("button:main.sources");
-	CONNECTS(toggleSourcesBtn, toggled, [this](Gtk::ToggleButton* b) {
-		m_notebook->set_visible(b->get_active());
-	});
-	CONNECT(m_addButton, clicked, [this] { openSources(); });
-	CONNECT(m_addButtonMenu, clicked, [this] { m_pasteItem->set_sensitive(m_clipboard->wait_is_image_available()); });
-	CONNECT(m_pasteItem, activate, [this] { pasteClipboard(); });
-	CONNECT(MAIN->getWidget("menuitem:sources.images.screenshot").as<Gtk::MenuItem>(), activate, [this] { takeScreenshot(); });
-	CONNECT(m_removeButton, clicked, [this] { removeSource(false); });
-	CONNECT(m_deleteButton, clicked, [this] { removeSource(true); });
-	CONNECT(m_clearButton, clicked, [this] { clearSources(); });
-	m_connectionSelectionChanged = CONNECT(m_listView->get_selection(), changed, [this] { selectionChanged(); });
+	CONNECT(ui.buttonSources, toggled, [this] { ui.notebookSources->set_visible(ui.buttonSources->get_active()); });
+	CONNECT(ui.buttonSourcesAdd, clicked, [this] { openSources(); });
+	CONNECT(ui.menubuttonSourcesAdd, clicked, [this] { ui.menuitemSourcesPaste->set_sensitive(m_clipboard->wait_is_image_available()); });
+	CONNECT(ui.menuitemSourcesPaste, activate, [this] { pasteClipboard(); });
+	CONNECT(ui.menuitemSourcesScreenshot, activate, [this] { takeScreenshot(); });
+	CONNECT(ui.buttonSourcesRemove, clicked, [this] { removeSource(false); });
+	CONNECT(ui.buttonSourcesDelete, clicked, [this] { removeSource(true); });
+	CONNECT(ui.buttonSourcesClear, clicked, [this] { clearSources(); });
+	m_connectionSelectionChanged = CONNECT(ui.treeviewSources->get_selection(), changed, [this] { selectionChanged(); });
 	CONNECT(recentChooser, item_activated, [this, recentChooser] { addSources({Gio::File::create_for_uri(recentChooser->get_current_uri())}); });
 
 	// Handle drops on the scrolled window
-	Gtk::ScrolledWindow* scollWin = MAIN->getWidget("scrollwin:sources.images");
-	scollWin->drag_dest_set({Gtk::TargetEntry("text/uri-list")}, Gtk::DEST_DEFAULT_MOTION | Gtk::DEST_DEFAULT_DROP, Gdk::ACTION_COPY | Gdk::ACTION_MOVE);
-	CONNECT(scollWin, drag_data_received, sigc::ptr_fun(Utils::handle_drag_drop));
+	ui.scrollwinSources->drag_dest_set({Gtk::TargetEntry("text/uri-list")}, Gtk::DEST_DEFAULT_MOTION | Gtk::DEST_DEFAULT_DROP, Gdk::ACTION_COPY | Gdk::ACTION_MOVE);
+	CONNECT(ui.scrollwinSources, drag_data_received, sigc::ptr_fun(Utils::handle_drag_drop));
 }
 
 SourceManager::~SourceManager() {
 	clearSources();
 }
 
-void SourceManager::addSources(const std::vector<Glib::RefPtr<Gio::File>>& files) {
+int SourceManager::addSources(const std::vector<Glib::RefPtr<Gio::File>>& files, bool suppressTextWarning) {
 	Glib::ustring failed;
-	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(m_listView->get_model());
+	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(ui.treeviewSources->get_model());
 	Gtk::TreeIter it = store->children().end();
+	int added = 0;
+	std::vector<Glib::ustring> filesWithText;
 	for(Glib::RefPtr<Gio::File> file : files) {
 		if(!file->query_exists()) {
-			failed += "\n\t" + file->get_path();
+			failed += "\n " + file->get_path();
 			continue;
 		}
 		bool contains = false;
 		for(const Gtk::TreeModel::Row& row : store->children()) {
 			if(row.get_value(m_listViewCols.source)->file->get_uri() == file->get_uri()) {
 				contains = true;
+				it = row;
 				break;
 			}
 		}
 		if(contains) {
+			++added;
+			continue;
+		}
+		Source* source = new Source(file, file->get_basename(), file->monitor_file(Gio::FILE_MONITOR_SEND_MOVED), false);
+		std::string filename = file->get_path();
+#ifdef G_OS_WIN32
+		if(Glib::ustring(filename.substr(filename.length() - 4)).lowercase() == ".pdf" && !checkPdfSource(source, filesWithText)) {
+#else
+		if(Utils::get_content_type(filename) == "application/pdf" && !checkPdfSource(source, filesWithText)) {
+#endif
+			delete source;
 			continue;
 		}
 		it = store->append();
 		it->set_value(m_listViewCols.filename, file->get_basename());
-		Source* source = new Source(file, file->get_basename(), file->monitor_file(Gio::FILE_MONITOR_SEND_MOVED), false);
 		it->set_value(m_listViewCols.source, source);
 		it->set_value(m_listViewCols.path, file->get_path());
 		CONNECT(source->monitor, changed, sigc::bind(sigc::mem_fun(*this, &SourceManager::fileChanged), it));
 
 		Gtk::RecentManager::get_default()->add_item(file->get_uri());
+		++added;
 	}
+	if(!suppressTextWarning && !filesWithText.empty()) {
+		Utils::message_dialog(Gtk::MESSAGE_INFO, _("PDFs with text"), Glib::ustring::compose(_("These PDF files already contain text:\n%1"), Utils::string_join(filesWithText, "\n")));
+	}
+	m_connectionSelectionChanged.block(true);
+	ui.treeviewSources->get_selection()->unselect_all();
+	m_connectionSelectionChanged.block(false);
 	if(it) {
-		m_connectionSelectionChanged.block(true);
-		m_listView->get_selection()->unselect_all();
-		m_connectionSelectionChanged.block(false);
-		m_listView->get_selection()->select(it);
+		ui.treeviewSources->get_selection()->select(it);
 	}
 	if(!failed.empty()) {
 		Utils::message_dialog(Gtk::MESSAGE_ERROR, _("Unable to open files"), Glib::ustring::compose(_("The following files could not be opened:%1"), failed));
 	}
+	return added;
+}
+
+bool SourceManager::checkPdfSource(Source* source, std::vector<Glib::ustring>& filesWithText) const {
+	GError* err = nullptr;
+	std::string filename = source->file->get_path();
+	PopplerDocument* document = poppler_document_new_from_file(Glib::filename_to_uri(filename).c_str(), 0, &err);
+
+	// Unlock if necessary
+	if(err && g_error_matches (err, POPPLER_ERROR, POPPLER_ERROR_ENCRYPTED)) {
+		g_error_free(err);
+		err = nullptr;
+		ui.labelPdfpassword->set_text(Glib::ustring::compose(_("Enter password for file '%1':"), source->file->get_basename()));
+		ui.entryPdfpassword->set_text("");
+		while(true) {
+			ui.entryPdfpassword->select_region(0, -1);
+			ui.entryPdfpassword->grab_focus();
+			int response = ui.dialogPdfpassword->run();
+			ui.dialogPdfpassword->hide();
+			if(response != Gtk::RESPONSE_OK) {
+				return false;
+			}
+			Glib::ustring pass = ui.entryPdfpassword->get_text();
+			document = poppler_document_new_from_file(Glib::filename_to_uri(filename).c_str(), pass.c_str(), &err);
+			if(!err) {
+				source->password = pass;
+				break;
+			}
+			g_error_free(err);
+			err = nullptr;
+		}
+	}
+
+	// Check whether the PDF already contains text
+	for (int i = 0, n = poppler_document_get_n_pages(document); i < n; ++i) {
+		PopplerPage* poppage = poppler_document_get_page(document, i);
+		bool hasText = poppler_page_get_text(poppage) != nullptr;
+		g_object_unref(poppage);
+		if(hasText) {
+			filesWithText.push_back(filename);
+			break;
+		}
+	}
+
+	// Extract document metadata
+	source->author = poppler_document_get_author(document) ? poppler_document_get_author(document) : "";
+	source->creator = poppler_document_get_creator(document) ? poppler_document_get_creator(document) : "";
+	source->keywords = poppler_document_get_keywords(document) ? poppler_document_get_keywords(document) : "";
+	source->producer = poppler_document_get_producer(document) ? poppler_document_get_producer(document) : "";
+	source->title = poppler_document_get_title(document) ? poppler_document_get_title(document) : "";
+	source->subject = poppler_document_get_subject(document) ? poppler_document_get_subject(document) : "";
+	poppler_document_get_pdf_version(document, &source->pdfVersionMajor, &source->pdfVersionMinor);
+
+	return true;
 }
 
 std::vector<Source*> SourceManager::getSelectedSources() const {
 	std::vector<Source*> selectedSources;
-	for(const Gtk::TreeModel::Path& path : m_listView->get_selection()->get_selected_rows()) {
-		selectedSources.push_back(m_listView->get_model()->get_iter(path)->get_value(m_listViewCols.source));
+	for(const Gtk::TreeModel::Path& path : ui.treeviewSources->get_selection()->get_selected_rows()) {
+		selectedSources.push_back(ui.treeviewSources->get_model()->get_iter(path)->get_value(m_listViewCols.source));
 	}
 	return selectedSources;
 }
@@ -134,7 +197,9 @@ void SourceManager::openSources() {
 	filter.name = _("Images and PDFs");
 	filter.mime_types.push_back("application/pdf");
 	filter.patterns.push_back("*.pdf");
-	addSources(FileDialogs::open_dialog(_("Select Files"), initialFolder, filter, true));
+	filter.mime_types.push_back("image/vnd.djvu");
+	filter.patterns.push_back("*.djvu");
+	addSources(FileDialogs::open_dialog(_("Select Files"), initialFolder, "sourcedir", filter, true));
 }
 
 void SourceManager::pasteClipboard() {
@@ -164,7 +229,7 @@ void SourceManager::takeScreenshot() {
 	savePixbuf(pixbuf, displayname);
 }
 
-void SourceManager::savePixbuf(const Glib::RefPtr<Gdk::Pixbuf> &pixbuf, const std::string &displayname) {
+void SourceManager::savePixbuf(const Glib::RefPtr<Gdk::Pixbuf>& pixbuf, const std::string& displayname) {
 	MAIN->pushState(MainWindow::State::Busy, _("Saving image..."));
 	std::string filename;
 	try {
@@ -177,7 +242,7 @@ void SourceManager::savePixbuf(const Glib::RefPtr<Gdk::Pixbuf> &pixbuf, const st
 		return;
 	}
 	MAIN->popState();
-	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(m_listView->get_model());
+	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(ui.treeviewSources->get_model());
 	Glib::RefPtr<Gio::File> file = Gio::File::create_for_path(filename);
 	Source* source = new Source(file, displayname, file->monitor_file(Gio::FILE_MONITOR_SEND_MOVED), true);
 	Gtk::TreeIter it = store->append();
@@ -186,29 +251,29 @@ void SourceManager::savePixbuf(const Glib::RefPtr<Gdk::Pixbuf> &pixbuf, const st
 	it->set_value(m_listViewCols.source, source);
 	CONNECT(source->monitor, changed, sigc::bind(sigc::mem_fun(*this, &SourceManager::fileChanged), it));
 	m_connectionSelectionChanged.block(true);
-	m_listView->get_selection()->unselect_all();
+	ui.treeviewSources->get_selection()->unselect_all();
 	m_connectionSelectionChanged.block(false);
-	m_listView->get_selection()->select(it);
+	ui.treeviewSources->get_selection()->select(it);
 }
 
 void SourceManager::removeSource(bool deleteFile) {
 	std::string paths;
-	for(const Gtk::TreeModel::Path& path : m_listView->get_selection()->get_selected_rows()) {
-		paths += std::string("\n") + m_listView->get_model()->get_iter(path)->get_value(m_listViewCols.source)->file->get_path();
+	for(const Gtk::TreeModel::Path& path : ui.treeviewSources->get_selection()->get_selected_rows()) {
+		paths += std::string("\n") + ui.treeviewSources->get_model()->get_iter(path)->get_value(m_listViewCols.source)->file->get_path();
 	}
 	if(paths.empty()) {
 		return;
 	}
-	if(deleteFile && Utils::Button::Yes != Utils::question_dialog(_("Delete File?"), Glib::ustring::compose(_("The following files will be deleted:%1"), paths), Utils::Button::Yes|Utils::Button::No)) {
+	if(deleteFile && Utils::Button::Yes != Utils::question_dialog(_("Delete File?"), Glib::ustring::compose(_("The following files will be deleted:%1"), paths), Utils::Button::Yes | Utils::Button::No)) {
 		return;
 	}
 	// Avoid multiple sourceChanged emissions when removing items
 	m_connectionSelectionChanged.block(true);
-	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(m_listView->get_model());
+	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(ui.treeviewSources->get_model());
 	Gtk::TreeIter it;
-	std::vector<Gtk::TreeModel::Path> selected = m_listView->get_selection()->get_selected_rows();
+	std::vector<Gtk::TreeModel::Path> selected = ui.treeviewSources->get_selection()->get_selected_rows();
 	while(!selected.empty()) {
-		it = m_listView->get_model()->get_iter(selected.back());
+		it = ui.treeviewSources->get_model()->get_iter(selected.back());
 		selected.pop_back();
 		Source* source = it->get_value(m_listViewCols.source);
 		if(deleteFile || source->isTemp) {
@@ -221,14 +286,14 @@ void SourceManager::removeSource(bool deleteFile) {
 		it = --store->children().end();
 	}
 	if(it) {
-		m_listView->get_selection()->select(it);
+		ui.treeviewSources->get_selection()->select(it);
 	}
 	m_connectionSelectionChanged.block(false);
 	selectionChanged();
 }
 
 void SourceManager::clearSources() {
-	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(m_listView->get_model());
+	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(ui.treeviewSources->get_model());
 	for(const Gtk::TreeModel::Row& row : store->children()) {
 		Source* source = row.get_value(m_listViewCols.source);
 		if(source->isTemp) {
@@ -243,23 +308,24 @@ void SourceManager::clearSources() {
 }
 
 void SourceManager::selectionChanged() {
-	bool enabled = !m_listView->get_selection()->get_selected_rows().empty();
-	m_removeButton->set_sensitive(enabled);
-	m_deleteButton->set_sensitive(enabled);
-	m_clearButton->set_sensitive(enabled);
+	bool enabled = !ui.treeviewSources->get_selection()->get_selected_rows().empty();
+	ui.buttonSourcesRemove->set_sensitive(enabled);
+	ui.buttonSourcesDelete->set_sensitive(enabled);
+	ui.buttonSourcesClear->set_sensitive(enabled);
 	m_signal_sourceChanged.emit();
 }
 
 void SourceManager::fileChanged(const Glib::RefPtr<Gio::File>& file, const Glib::RefPtr<Gio::File>& otherFile, Gio::FileMonitorEvent event, Gtk::TreeIter it) {
-	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(m_listView->get_model());
+	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(ui.treeviewSources->get_model());
 	Source* source = it->get_value(m_listViewCols.source);
 	if(event == Gio::FILE_MONITOR_EVENT_MOVED) {
 		source->file = otherFile;
+		source->displayname = otherFile->get_basename();
 		source->monitor = otherFile->monitor_file(Gio::FILE_MONITOR_SEND_MOVED);
 		it->set_value(m_listViewCols.filename, otherFile->get_basename());
 		it->set_value(m_listViewCols.path, otherFile->get_path());
 		CONNECT(source->monitor, changed, sigc::bind(sigc::mem_fun(*this, &SourceManager::fileChanged), it));
-		if(it == m_listView->get_selection()->get_selected()) {
+		if(ui.treeviewSources->get_selection()->is_selected(it)) {
 			m_signal_sourceChanged.emit();
 		}
 	} else if(event == Gio::FILE_MONITOR_EVENT_DELETED) {
@@ -270,7 +336,7 @@ void SourceManager::fileChanged(const Glib::RefPtr<Gio::File>& file, const Glib:
 			it = store->children().begin();
 		}
 		if(it) {
-			m_listView->get_selection()->select(it);
+			ui.treeviewSources->get_selection()->select(it);
 		}
 	}
 }
