@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Acquirer.cc
- * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -18,12 +18,12 @@
  */
 
 #include <QDir>
-#include <QFileDialog>
 #include <QImageReader>
 #include <QThread>
 
 #include "Acquirer.hh"
-#include "Config.hh"
+#include "ConfigSettings.hh"
+#include "FileDialogs.hh"
 #include "MainWindow.hh"
 #include "Utils.hh"
 
@@ -57,12 +57,17 @@ Acquirer::Acquirer(const UI_MainWindow& _ui)
 	connect(m_scanner, SIGNAL(scanStateChanged(Scanner::State)), this, SLOT(setScanState(Scanner::State)));
 	connect(m_scanner, SIGNAL(pageAvailable(QString)), this, SIGNAL(scanPageAvailable(QString)));
 
-	MAIN->getConfig()->addSetting(new ComboSetting("scanres", ui.comboBoxScanResolution, 2));
-	MAIN->getConfig()->addSetting(new ComboSetting("scanmode", ui.comboBoxScanMode, 0));
-	MAIN->getConfig()->addSetting(new ComboSetting("scandev", ui.comboBoxScanDevice, 0));
-	MAIN->getConfig()->addSetting(new VarSetting<QString>("scanoutput", QDir(Utils::documentsFolder()).absoluteFilePath(_("scan.png"))));
+	ADD_SETTING(ComboSetting("scanres", ui.comboBoxScanResolution, 2));
+	ADD_SETTING(ComboSetting("scanmode", ui.comboBoxScanMode, 0));
+	ADD_SETTING(ComboSetting("scandev", ui.comboBoxScanDevice, 0));
+	ADD_SETTING(ComboSetting("scansource", ui.comboBoxScanSource, 0));
+#ifdef Q_OS_WIN32
+	ui.labelScanSource->setVisible(false);
+	ui.comboBoxScanSource->setVisible(false);
+#endif
 
-	m_outputPath = MAIN->getConfig()->getSetting<VarSetting<QString>>("scanoutput")->getValue();
+	QString sourcedir = ConfigSettings::get<VarSetting<QString>>("sourcedir")->getValue();
+	m_outputPath = QDir(sourcedir.isEmpty() ? Utils::documentsFolder() : sourcedir).absoluteFilePath(_("scan.png"));
 	genOutputPath();
 	m_scanner->init();
 }
@@ -78,7 +83,7 @@ void Acquirer::selectOutputPath() {
 		formats.insert(QString("*.%1").arg(QString(format).toLower()));
 	}
 	QString filter = QString("%1 (%2)").arg(_("Images")).arg(QStringList(formats.toList()).join(" "));
-	QString filename = QFileDialog::getSaveFileName(MAIN, _("Choose Output Filename..."), m_outputPath, filter);
+	QString filename = FileDialogs::saveDialog(_("Choose Output Filename..."), m_outputPath, "sourcedir", filter);
 	if(!filename.isEmpty()) {
 		m_outputPath = filename;
 		genOutputPath();
@@ -88,8 +93,7 @@ void Acquirer::selectOutputPath() {
 void Acquirer::genOutputPath() {
 	m_outputPath = Utils::makeOutputFilename(m_outputPath);
 	ui.lineEditScanOutput->setText(m_outputPath);
-	ui.lineEditScanOutput->setToolTip(m_outputPath);;
-	MAIN->getConfig()->getSetting<VarSetting<QString>>("scanoutput")->setValue(m_outputPath);
+	ui.lineEditScanOutput->setToolTip(m_outputPath);
 }
 
 void Acquirer::scanInitFailed() {
@@ -98,7 +102,7 @@ void Acquirer::scanInitFailed() {
 	ui.toolButtonScanDevicesRefresh->setEnabled(false);
 }
 
-void Acquirer::scanFailed(const QString &msg) {
+void Acquirer::scanFailed(const QString& msg) {
 	ui.labelScanMessage->setText(QString("<span style=\"color:#FF0000;\">%1: %2.</span>").arg(_("Scan failed")).arg(msg));
 }
 
@@ -131,11 +135,12 @@ void Acquirer::startScan() {
 	ui.pushButtonScanCancel->setVisible(true);
 	ui.labelScanMessage->setText(_("Starting scan..."));
 
-	double dpi[] = {75., 100., 200., 300., 600., 1200.};
+	double dpi[] = {75.0, 100.0, 200.0, 300.0, 600.0, 1200.0};
 	Scanner::ScanMode modes[] = {Scanner::ScanMode::GRAY, Scanner::ScanMode::COLOR};
+	Scanner::ScanType types[] = {Scanner::ScanType::SINGLE, Scanner::ScanType::ADF_FRONT, Scanner::ScanType::ADF_BACK, Scanner::ScanType::ADF_BOTH};
 	genOutputPath();
 	QString device = ui.comboBoxScanDevice->itemData(ui.comboBoxScanDevice->currentIndex()).toString();
-	Scanner::Params params = {device, m_outputPath, dpi[ui.comboBoxScanResolution->currentIndex()], modes[ui.comboBoxScanMode->currentIndex()], 8, Scanner::ScanType::SINGLE, 0, 0};
+	Scanner::Params params = {device, m_outputPath, dpi[ui.comboBoxScanResolution->currentIndex()], modes[ui.comboBoxScanMode->currentIndex()], 8, types[ui.comboBoxScanSource->currentIndex()], 0, 0};
 	m_scanner->scan(params);
 	genOutputPath(); // Prepare for next
 }
