@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Displayer.cc
- * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -18,12 +18,12 @@
  */
 
 #include "MainWindow.hh"
-#include "Config.hh"
 #include "Displayer.hh"
 #include "DisplayRenderer.hh"
 #include "SourceManager.hh"
 #include "Utils.hh"
 
+#include <cmath>
 #include <QFileDialog>
 #include <QGraphicsPixmapItem>
 #include <QGraphicsSceneDragDropEvent>
@@ -38,13 +38,13 @@ public:
 	using QGraphicsScene::QGraphicsScene;
 
 protected:
-	void dragEnterEvent(QGraphicsSceneDragDropEvent *event) {
+	void dragEnterEvent(QGraphicsSceneDragDropEvent* event) {
 		if(Utils::handleSourceDragEvent(event->mimeData())) {
 			event->acceptProposedAction();
 		}
 	}
-	void dragMoveEvent(QGraphicsSceneDragDropEvent *event) {}
-	void dropEvent(QGraphicsSceneDragDropEvent *event) {
+	void dragMoveEvent(QGraphicsSceneDragDropEvent* event) {}
+	void dropEvent(QGraphicsSceneDragDropEvent* event) {
 		Utils::handleSourceDropEvent(event->mimeData());
 	}
 };
@@ -63,18 +63,18 @@ Displayer::Displayer(const UI_MainWindow& _ui, QWidget* parent)
 	m_renderTimer.setSingleShot(true);
 	m_scaleTimer.setSingleShot(true);
 
-	ui.actionRotateLeft->setData(270.);
-	ui.actionRotateRight->setData(90.);
+	ui.actionRotateLeft->setData(270.0);
+	ui.actionRotateRight->setData(90.0);
 
 	connect(ui.menuRotation, SIGNAL(triggered(QAction*)), this, SLOT(setRotateMode(QAction*)));
 	connect(ui.actionRotateLeft, SIGNAL(triggered()), this, SLOT(rotate90()));
 	connect(ui.actionRotateRight, SIGNAL(triggered()), this, SLOT(rotate90()));
 	connect(ui.spinBoxRotation, SIGNAL(valueChanged(double)), this, SLOT(setAngle(double)));
-	connect(ui.spinBoxPage, SIGNAL(valueChanged(int)), this, SLOT(setCurrentPage(int)));
-	connect(ui.spinBoxBrightness, SIGNAL(valueChanged(int)), this, SLOT(brightnessChanged()));
-	connect(ui.spinBoxContrast, SIGNAL(valueChanged(int)), this, SLOT(contrastChanged()));
-	connect(ui.spinBoxResolution, SIGNAL(valueChanged(int)), this, SLOT(resolutionChanged()));
-	connect(ui.checkBoxInvertColors, SIGNAL(toggled(bool)), this, SLOT(invertColorsChanged()));
+	connect(ui.spinBoxPage, SIGNAL(valueChanged(int)), this, SLOT(queueRenderImage()));
+	connect(ui.spinBoxBrightness, SIGNAL(valueChanged(int)), this, SLOT(queueRenderImage()));
+	connect(ui.spinBoxContrast, SIGNAL(valueChanged(int)), this, SLOT(queueRenderImage()));
+	connect(ui.spinBoxResolution, SIGNAL(valueChanged(int)), this, SLOT(queueRenderImage()));
+	connect(ui.checkBoxInvertColors, SIGNAL(toggled(bool)), this, SLOT(queueRenderImage()));
 	connect(ui.actionZoomIn, SIGNAL(triggered()), this, SLOT(zoomIn()));
 	connect(ui.actionZoomOut, SIGNAL(triggered()), this, SLOT(zoomOut()));
 	connect(ui.actionBestFit, SIGNAL(triggered()), this, SLOT(zoomFit()));
@@ -88,68 +88,6 @@ Displayer::~Displayer() {
 	delete m_scene;
 }
 
-bool Displayer::setCurrentPage(int page) {
-	ui.spinBoxPage->setEnabled(false);
-	if(m_sources.isEmpty()) {
-		return false;
-	}
-	if(m_tool) {
-		m_tool->pageChanged();
-	}
-	Source* source = m_pageMap[page].first;
-	if(source != m_currentSource) {
-		sendScaleRequest({ScaleRequest::Abort});
-		sendScaleRequest({ScaleRequest::Quit});
-		while(m_scaleThread.isRunning()) {
-			QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
-		}
-		delete m_renderer;
-		if(source->path.endsWith(".pdf", Qt::CaseInsensitive)) {
-			m_renderer = new PDFRenderer(source->path);
-			if(source->resolution == -1) source->resolution = 300;
-		} else {
-			m_renderer = new ImageRenderer(source->path);
-			if(source->resolution == -1) source->resolution = 100;
-		}
-
-		Utils::setSpinBlocked(ui.spinBoxBrightness, source->brightness);
-		Utils::setSpinBlocked(ui.spinBoxContrast, source->contrast);
-		Utils::setSpinBlocked(ui.spinBoxResolution, source->resolution);
-		ui.checkBoxInvertColors->blockSignals(true);
-		ui.checkBoxInvertColors->setChecked(source->invert);
-		ui.checkBoxInvertColors->blockSignals(false);
-		m_currentSource = source;
-		m_scaleThread.start();
-	}
-	Utils::setSpinBlocked(ui.spinBoxRotation, source->angle[m_pageMap[page].second - 1]);
-	Utils::setSpinBlocked(ui.spinBoxPage, page);
-
-	bool result = renderImage();
-	ui.spinBoxPage->setEnabled(true);
-	return result;
-}
-
-int Displayer::getCurrentPage() const {
-	return ui.spinBoxPage->value();
-}
-
-double Displayer::getCurrentAngle() const {
-	return ui.spinBoxRotation->value();
-}
-
-int Displayer::getCurrentResolution() const {
-	return ui.spinBoxResolution->value();
-}
-
-QString Displayer::getCurrentImage(int& page) const {
-	page = m_pageMap[ui.spinBoxPage->value()].second;
-	return m_pageMap[ui.spinBoxPage->value()].first ? m_pageMap[ui.spinBoxPage->value()].first->path : "";
-}
-
-int Displayer::getNPages() const {
-	return ui.spinBoxPage->maximum();
-}
-
 bool Displayer::setSources(QList<Source*> sources) {
 	if(sources == m_sources) {
 		return true;
@@ -162,12 +100,13 @@ bool Displayer::setSources(QList<Source*> sources) {
 		while(m_scaleThread.isRunning()) {
 			QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
 		}
+		m_scaleRequests.clear();
 	}
 	if(m_tool) {
 		m_tool->reset();
 	}
 	m_renderTimer.stop();
-	m_scene->clear();
+	m_scene->removeItem(m_imageItem);
 	delete m_renderer;
 	m_renderer = nullptr;
 	m_currentSource = nullptr;
@@ -176,13 +115,17 @@ bool Displayer::setSources(QList<Source*> sources) {
 	m_pixmap = QPixmap();
 	m_imageItem = nullptr;
 	ui.actionBestFit->setChecked(true);
-	ui.spinBoxPage->setEnabled(false);
+	ui.actionPage->setVisible(false);
+	ui.spinBoxPage->blockSignals(true);
 	ui.spinBoxPage->setRange(1, 1);
-	ui.spinBoxRotation->setValue(0.);
-	ui.spinBoxBrightness->setValue(0);
-	ui.spinBoxContrast->setValue(0);
-	ui.spinBoxResolution->setValue(100);
+	ui.spinBoxPage->blockSignals(false);
+	Utils::setSpinBlocked(ui.spinBoxRotation, 0);
+	Utils::setSpinBlocked(ui.spinBoxBrightness, 0);
+	Utils::setSpinBlocked(ui.spinBoxContrast, 0);
+	Utils::setSpinBlocked(ui.spinBoxResolution, 100);
+	ui.checkBoxInvertColors->blockSignals(true);
 	ui.checkBoxInvertColors->setChecked(false);
+	ui.checkBoxInvertColors->blockSignals(false);
 	ui.actionBestFit->setChecked(true);
 	ui.actionOriginalSize->setChecked(false);
 	ui.actionZoomIn->setEnabled(true);
@@ -201,7 +144,9 @@ bool Displayer::setSources(QList<Source*> sources) {
 	for(Source* source : m_sources) {
 		DisplayRenderer* renderer;
 		if(source->path.endsWith(".pdf", Qt::CaseInsensitive)) {
-			renderer = new PDFRenderer(source->path);
+			renderer = new PDFRenderer(source->path, source->password);
+		} else if(source->path.endsWith(".djvu", Qt::CaseInsensitive)) {
+			renderer = new DJVURenderer(source->path);
 		} else {
 			renderer = new ImageRenderer(source->path);
 		}
@@ -221,12 +166,10 @@ bool Displayer::setSources(QList<Source*> sources) {
 	ui.spinBoxPage->setMaximum(page);
 	ui.spinBoxPage->blockSignals(false);
 	ui.actionPage->setVisible(page > 1);
-	setCursor(Qt::CrossCursor);
 	m_imageItem = new QGraphicsPixmapItem();
 	m_imageItem->setTransformationMode(Qt::SmoothTransformation);
 	m_scene->addItem(m_imageItem);
-	m_scaleThread.start();
-	if(!setCurrentPage(1)) {
+	if(!renderImage()) {
 		Q_ASSERT(m_currentSource);
 		QMessageBox::critical(this, _("Failed to load image"), _("The file might not be an image or be corrupt:\n%1").arg(m_currentSource->displayname));
 		setSources(QList<Source*>());
@@ -235,35 +178,96 @@ bool Displayer::setSources(QList<Source*> sources) {
 	return true;
 }
 
-bool Displayer::hasMultipleOCRAreas() {
-	return m_tool->hasMultipleOCRAreas();
+bool Displayer::setup(const int* page, const int* resolution, const double* angle) {
+	bool changed = false;
+	if(page) {
+		changed |= *page != ui.spinBoxPage->value();
+		Utils::setSpinBlocked(ui.spinBoxPage, *page);
+	}
+	if(resolution) {
+		changed |= *resolution != ui.spinBoxResolution->value();
+		Utils::setSpinBlocked(ui.spinBoxResolution, *resolution);
+	}
+	if(changed && !renderImage()) {
+		return false;
+	}
+	if(angle) {
+		setAngle(*angle);
+	}
+	return true;
 }
 
-QList<QImage> Displayer::getOCRAreas() {
-	return m_tool->getOCRAreas();
-}
-
-bool Displayer::allowAutodetectOCRAreas() const {
-	return m_tool->allowAutodetectOCRAreas();
-}
-
-void Displayer::autodetectOCRAreas() {
-	m_tool->autodetectOCRAreas();
+void Displayer::queueRenderImage() {
+	m_renderTimer.start(500);
 }
 
 bool Displayer::renderImage() {
-	sendScaleRequest({ScaleRequest::Abort});
-	if(m_currentSource->resolution != ui.spinBoxResolution->value()) {
-		double factor = double(ui.spinBoxResolution->value()) / double(m_currentSource->resolution);
-		if(m_tool) {
-			m_tool->resolutionChanged(factor);
-		}
+	if(m_sources.isEmpty()) {
+		return false;
 	}
+	int page = ui.spinBoxPage->value();
+
+	// Set current source according to selected page
+	Source* source = m_pageMap[page].first;
+	if(!source) {
+		return false;
+	}
+
+	int oldResolution = m_currentSource ? m_currentSource->resolution : -1;
+	int oldPage = m_currentSource ? m_currentSource->page : -1;
+	Source* oldSource = m_currentSource;
+
+	if(source != m_currentSource) {
+		sendScaleRequest({ScaleRequest::Abort});
+		sendScaleRequest({ScaleRequest::Quit});
+		while(m_scaleThread.isRunning()) {
+			QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+		}
+		m_scaleRequests.clear();
+		delete m_renderer;
+		if(source->path.endsWith(".pdf", Qt::CaseInsensitive)) {
+			m_renderer = new PDFRenderer(source->path, source->password);
+			if(source->resolution == -1) { source->resolution = 300; }
+		} else if(source->path.endsWith(".djvu", Qt::CaseInsensitive)) {
+			m_renderer = new DJVURenderer(source->path);
+			if(source->resolution == -1) { source->resolution = 300; }
+		} else {
+			m_renderer = new ImageRenderer(source->path);
+			if(source->resolution == -1) { source->resolution = 100; }
+		}
+
+		Utils::setSpinBlocked(ui.spinBoxResolution, source->resolution);
+		Utils::setSpinBlocked(ui.spinBoxBrightness, source->brightness);
+		Utils::setSpinBlocked(ui.spinBoxContrast, source->contrast);
+		ui.checkBoxInvertColors->blockSignals(true);
+		ui.checkBoxInvertColors->setChecked(source->invert);
+		ui.checkBoxInvertColors->blockSignals(false);
+		m_currentSource = source;
+		m_scaleThread.start();
+	}
+
+	// Update source struct
 	m_currentSource->page = m_pageMap[ui.spinBoxPage->value()].second;
 	m_currentSource->brightness = ui.spinBoxBrightness->value();
 	m_currentSource->contrast = ui.spinBoxContrast->value();
 	m_currentSource->resolution = ui.spinBoxResolution->value();
 	m_currentSource->invert = ui.checkBoxInvertColors->isChecked();
+
+	// Notify tools about changes
+	if(m_tool) {
+		if(m_currentSource != oldSource || m_currentSource->page != oldPage) {
+			m_tool->pageChanged();
+		}
+		if(oldResolution != m_currentSource->resolution) {
+			double factor = double(m_currentSource->resolution) / double(oldResolution);
+			m_tool->resolutionChanged(factor);
+		}
+	}
+
+	Utils::setSpinBlocked(ui.spinBoxRotation, m_currentSource->angle[m_currentSource->page - 1]);
+
+	// Render new image
+	sendScaleRequest({ScaleRequest::Abort});
 	QImage image = m_renderer->render(m_currentSource->page, m_currentSource->resolution);
 	if(image.isNull()) {
 		return false;
@@ -284,6 +288,43 @@ bool Displayer::renderImage() {
 	return true;
 }
 
+int Displayer::getCurrentPage() const {
+	return ui.spinBoxPage->value();
+}
+
+int Displayer::getNPages() const {
+	return ui.spinBoxPage->maximum();
+}
+
+double Displayer::getCurrentAngle() const {
+	return ui.spinBoxRotation->value();
+}
+
+int Displayer::getCurrentResolution() const {
+	return ui.spinBoxResolution->value();
+}
+
+QString Displayer::getCurrentImage(int& page) const {
+	page = m_pageMap[ui.spinBoxPage->value()].second;
+	return m_pageMap[ui.spinBoxPage->value()].first ? m_pageMap[ui.spinBoxPage->value()].first->path : "";
+}
+
+bool Displayer::hasMultipleOCRAreas() {
+	return m_tool->hasMultipleOCRAreas();
+}
+
+QList<QImage> Displayer::getOCRAreas() {
+	return m_tool->getOCRAreas();
+}
+
+bool Displayer::allowAutodetectOCRAreas() const {
+	return m_tool->allowAutodetectOCRAreas();
+}
+
+void Displayer::autodetectOCRAreas() {
+	m_tool->autodetectOCRAreas();
+}
+
 void Displayer::setZoom(Zoom action, ViewportAnchor anchor) {
 	if(!m_imageItem) {
 		return;
@@ -292,14 +333,14 @@ void Displayer::setZoom(Zoom action, ViewportAnchor anchor) {
 	setUpdatesEnabled(false);
 
 	QRectF bb = m_imageItem->sceneBoundingRect();
-	double fit = qMin(viewport()->width() / bb.width(), viewport()->height() / bb.height());
+	double fit = std::min(viewport()->width() / bb.width(), viewport()->height() / bb.height());
 
 	if(action == Zoom::Original) {
 		m_scale = 1.0;
 	} else if(action == Zoom::In) {
-		m_scale = qMin(10., m_scale * 1.25);
+		m_scale = std::min(10., m_scale * 1.25);
 	} else if(action == Zoom::Out) {
-		m_scale = qMax(0.05, m_scale * 0.8);
+		m_scale = std::max(0.05, m_scale * 0.8);
 	}
 	ui.actionBestFit->setChecked(false);
 	if(action == Zoom::Fit || (m_scale / fit >= 0.9 && m_scale / fit <= 1.09)) {
@@ -331,7 +372,7 @@ void Displayer::setZoom(Zoom action, ViewportAnchor anchor) {
 
 void Displayer::setAngle(double angle) {
 	if(m_imageItem) {
-		angle = angle < 0 ? angle + 360. : angle >= 360 ? angle - 360 : angle,
+		angle = angle < 0.0 ? angle + 360.0 : angle >= 360.0 ? angle - 360.0 : angle,
 		Utils::setSpinBlocked(ui.spinBoxRotation, angle);
 		int sourcePage = m_pageMap[getCurrentPage()].second;
 		double delta = angle - m_currentSource->angle[sourcePage - 1];
@@ -340,11 +381,13 @@ void Displayer::setAngle(double angle) {
 		} else if(delta != 0) {
 			for(int page : m_pageMap.keys()) {
 				auto pair = m_pageMap[page];
-				pair.first->angle[pair.second - 1] += delta;
+				double newangle = pair.first->angle[pair.second - 1] + delta;
+				newangle = newangle < 0.0 ? newangle + 360.0 : newangle >= 360.0 ? newangle - 360.0 : newangle,
+				pair.first->angle[pair.second - 1] = newangle;
 			}
 		}
 		m_imageItem->setRotation(angle);
-		if(m_tool) {
+		if(m_tool && delta != 0) {
 			m_tool->rotationChanged(delta);
 		}
 		m_scene->setSceneRect(m_imageItem->sceneBoundingRect());
@@ -354,59 +397,18 @@ void Displayer::setAngle(double angle) {
 	}
 }
 
-
-void Displayer::brightnessChanged() {
-	int brightness = ui.spinBoxBrightness->value();
-	for(int page : m_pageMap.keys()) {
-		m_pageMap[page].first->brightness = brightness;
-	}
-	queueRenderImage();
+void Displayer::rotate90() {
+	setAngle(ui.spinBoxRotation->value() + qobject_cast<QAction*>(QObject::sender())->data().toDouble());
 }
 
-void Displayer::contrastChanged() {
-	int contrast = ui.spinBoxBrightness->value();
-	for(int page : m_pageMap.keys()) {
-		m_pageMap[page].first->contrast = contrast;
-	}
-	queueRenderImage();
-}
-
-void Displayer::resolutionChanged() {
-	int resolution = ui.spinBoxResolution->value();
-	for(int page : m_pageMap.keys()) {
-		m_pageMap[page].first->resolution = resolution;
-	}
-	queueRenderImage();
-}
-
-void Displayer::invertColorsChanged() {
-	bool invert = ui.checkBoxInvertColors->isChecked();
-	for(int page : m_pageMap.keys()) {
-		m_pageMap[page].first->invert = invert;
-	}
-	queueRenderImage();
-}
-
-void Displayer::setResolution(int resolution) {
-	Utils::setSpinBlocked(ui.spinBoxResolution, resolution);
-	renderImage();
-}
-
-void Displayer::queueRenderImage() {
-	if(m_renderer) {
-		m_renderTimer.start(500);
-	}
-}
-
-void Displayer::resizeEvent(QResizeEvent *event) {
+void Displayer::resizeEvent(QResizeEvent* event) {
 	QGraphicsView::resizeEvent(event);
 	if(ui.actionBestFit->isChecked()) {
 		setZoom(Zoom::Fit);
 	}
 }
 
-void Displayer::keyPressEvent(QKeyEvent *event)
-{
+void Displayer::keyPressEvent(QKeyEvent* event) {
 	if(event->key() == Qt::Key_PageUp) {
 		ui.spinBoxPage->setValue(ui.spinBoxPage->value() - 1);
 		event->accept();
@@ -418,7 +420,7 @@ void Displayer::keyPressEvent(QKeyEvent *event)
 	}
 }
 
-void Displayer::mousePressEvent(QMouseEvent *event) {
+void Displayer::mousePressEvent(QMouseEvent* event) {
 	if(event->button() == Qt::MiddleButton) {
 		m_panPos = event->pos();
 	} else {
@@ -430,7 +432,7 @@ void Displayer::mousePressEvent(QMouseEvent *event) {
 	}
 }
 
-void Displayer::mouseMoveEvent(QMouseEvent *event) {
+void Displayer::mouseMoveEvent(QMouseEvent* event) {
 	if((event->buttons() & Qt::MiddleButton) == Qt::MiddleButton) {
 		QPoint delta = event->pos() - m_panPos;
 		horizontalScrollBar()->setValue( horizontalScrollBar()->value() - delta.x() );
@@ -445,7 +447,7 @@ void Displayer::mouseMoveEvent(QMouseEvent *event) {
 	}
 }
 
-void Displayer::mouseReleaseEvent(QMouseEvent *event) {
+void Displayer::mouseReleaseEvent(QMouseEvent* event) {
 	event->ignore();
 	QGraphicsView::mouseReleaseEvent(event);
 	if(!event->isAccepted() && m_tool && m_currentSource) {
@@ -453,7 +455,7 @@ void Displayer::mouseReleaseEvent(QMouseEvent *event) {
 	}
 }
 
-void Displayer::wheelEvent(QWheelEvent *event) {
+void Displayer::wheelEvent(QWheelEvent* event) {
 	if(event->modifiers() & Qt::ControlModifier) {
 		setZoom(event->delta() > 0 ? Zoom::In : Zoom::Out, QGraphicsView::AnchorUnderMouse);
 		event->accept();
@@ -470,24 +472,19 @@ void Displayer::wheelEvent(QWheelEvent *event) {
 	}
 }
 
-QPointF Displayer::mapToSceneClamped(const QPoint &p) const {
+QPointF Displayer::mapToSceneClamped(const QPoint& p) const {
 	QPointF q = mapToScene(p);
 	QRectF bb = m_imageItem->sceneBoundingRect();
-	q.rx() = qMin(qMax(bb.x(), q.x()), bb.x() + bb.width());
-	q.ry() = qMin(qMax(bb.y(), q.y()), bb.y() + bb.height());
+	q.rx() = std::min(std::max(bb.x(), q.x()), bb.x() + bb.width());
+	q.ry() = std::min(std::max(bb.y(), q.y()), bb.y() + bb.height());
 	return q;
 }
 
-void Displayer::setRotateMode(QAction *action) {
+void Displayer::setRotateMode(QAction* action) {
 	m_rotateMode = static_cast<RotateMode>(action->data().value<int>());
 	ui.toolButtonRotation->setIcon(action->icon());
 }
 
-void Displayer::rotate90() {
-	double angle = ui.spinBoxRotation->value() + qobject_cast<QAction*>(QObject::sender())->data().toDouble();
-	ui.spinBoxRotation->setValue(angle >= 360. ? angle - 360. : angle);
-}
-
 QImage Displayer::getImage(const QRectF& rect) {
 	QImage image(rect.width(), rect.height(), QImage::Format_RGB32);
 	image.fill(Qt::black);
@@ -514,6 +511,10 @@ QRectF Displayer::getSceneBoundingRect() const {
 	return transform.mapRect(rect);
 }
 
+void Displayer::scaleTimerElapsed() {
+	sendScaleRequest(m_pendingScaleRequest);
+}
+
 void Displayer::sendScaleRequest(const ScaleRequest& request) {
 	m_scaleTimer.stop();
 	m_scaleMutex.lock();
@@ -561,13 +562,13 @@ void Displayer::scaleThread() {
 	m_scaleMutex.unlock();
 }
 
-void Displayer::setScaledImage(const QImage &image, double scale) {
+void Displayer::setScaledImage(const QImage& image, double scale) {
 	m_scaleMutex.lock();
 	if(!m_scaleRequests.isEmpty() && m_scaleRequests.first().type == ScaleRequest::Abort) {
 		m_scaleRequests.removeFirst();
 	} else {
 		m_imageItem->setPixmap(QPixmap::fromImage(image));
-		m_imageItem->setScale(1.f / scale);
+		m_imageItem->setScale(1.0 / scale);
 		m_imageItem->setTransformOriginPoint(m_imageItem->boundingRect().center());
 		m_imageItem->setPos(m_imageItem->pos() - m_imageItem->sceneBoundingRect().center());
 	}
@@ -576,7 +577,7 @@ void Displayer::setScaledImage(const QImage &image, double scale) {
 
 ///////////////////////////////////////////////////////////////////////////////
 
-void DisplayerSelection::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) {
+void DisplayerSelection::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget) {
 	QColor c = QPalette().highlight().color();
 	setBrush(QColor(c.red(), c.green(), c.blue(), 63));
 	QPen pen;
@@ -589,15 +590,15 @@ void DisplayerSelection::paint(QPainter *painter, const QStyleOptionGraphicsItem
 	painter->setRenderHint(QPainter::Antialiasing, true);
 }
 
-void DisplayerSelection::hoverMoveEvent(QGraphicsSceneHoverEvent *event) {
+void DisplayerSelection::hoverMoveEvent(QGraphicsSceneHoverEvent* event) {
 	QPointF p = event->pos();
 	QRectF r = rect();
 	double tol = 10.0 / m_tool->getDisplayer()->getCurrentScale();
 
-	bool left = qAbs(r.x() - p.x()) < tol;
-	bool right = qAbs(r.x() + r.width() - p.x()) < tol;
-	bool top = qAbs(r.y() - p.y()) < tol;
-	bool bottom = qAbs(r.y() + r.height() - p.y()) < tol;
+	bool left = std::abs(r.x() - p.x()) < tol;
+	bool right = std::abs(r.x() + r.width() - p.x()) < tol;
+	bool top = std::abs(r.y() - p.y()) < tol;
+	bool bottom = std::abs(r.y() + r.height() - p.y()) < tol;
 
 	if((top && left) || (bottom && right)) {
 		setCursor(Qt::SizeFDiagCursor);
@@ -612,38 +613,43 @@ void DisplayerSelection::hoverMoveEvent(QGraphicsSceneHoverEvent *event) {
 	}
 }
 
-void DisplayerSelection::mousePressEvent(QGraphicsSceneMouseEvent *event) {
+void DisplayerSelection::mousePressEvent(QGraphicsSceneMouseEvent* event) {
 	QPointF p = event->pos();
 	double tol = 10.0 / m_tool->getDisplayer()->getCurrentScale();
 	m_resizeHandlers.clear();
-	m_resizeOffset = QPointF(0., 0.);
-	if(qAbs(m_point.x() - p.x()) < tol) { // pointx
+	m_resizeOffset = QPointF(0.0, 0.0);
+	if(std::abs(m_point.x() - p.x()) < tol) { // pointx
 		m_resizeHandlers.append(resizePointX);
 		m_resizeOffset.setX(event->pos().x() - m_point.x());
-	} else if(qAbs(m_anchor.x() - p.x()) < tol) { // anchorx
+	} else if(std::abs(m_anchor.x() - p.x()) < tol) { // anchorx
 		m_resizeHandlers.append(resizeAnchorX);
 		m_resizeOffset.setX(event->pos().x() - m_anchor.x());
 	}
-	if(qAbs(m_point.y() - p.y()) < tol) { // pointy
+	if(std::abs(m_point.y() - p.y()) < tol) { // pointy
 		m_resizeHandlers.append(resizePointY);
 		m_resizeOffset.setY(event->pos().y() - m_point.y());
-	} else if(qAbs(m_anchor.y() - p.y()) < tol) { // anchory
+	} else if(std::abs(m_anchor.y() - p.y()) < tol) { // anchory
 		m_resizeHandlers.append(resizeAnchorY);
 		m_resizeOffset.setY(event->pos().y() - m_anchor.y());
 	}
-	event->accept();
+	if(!m_resizeHandlers.empty()) {
+		event->accept();
+	} else {
+		event->ignore();
+	}
 }
 
-void DisplayerSelection::mouseMoveEvent(QGraphicsSceneMouseEvent *event) {
+void DisplayerSelection::mouseMoveEvent(QGraphicsSceneMouseEvent* event) {
 	QPointF p = event->pos() - m_resizeOffset;
 	QRectF bb = m_tool->getDisplayer()->getSceneBoundingRect();
-	p.rx() = qMin(qMax(bb.x(), p.x()), bb.x() + bb.width());
-	p.ry() = qMin(qMax(bb.y(), p.y()), bb.y() + bb.height());
+	p.rx() = std::min(std::max(bb.x(), p.x()), bb.x() + bb.width());
+	p.ry() = std::min(std::max(bb.y(), p.y()), bb.y() + bb.height());
 	if(!m_resizeHandlers.isEmpty()) {
 		for(const ResizeHandler& handler : m_resizeHandlers) {
 			handler(p, m_anchor, m_point);
 		}
-		setRect(QRectF(m_anchor, m_point).normalized());
+		QRectF newRect = QRectF(m_anchor, m_point).normalized().united(m_minRect);
+		setRect(newRect);
 		emit geometryChanged(rect());
 		event->accept();
 	} else {
