@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * DisplayerToolSelect.cc
- * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,13 +19,15 @@
 
 #include "DisplayerToolSelect.hh"
 #include "Displayer.hh"
+#include "FileDialogs.hh"
 #include "MainWindow.hh"
 #include "Recognizer.hh"
 #include "Utils.hh"
 
 #include <cmath>
+#define USE_STD_NAMESPACE
 #include <tesseract/baseapi.h>
-#include <QFileDialog>
+#undef USE_STD_NAMESPACE
 #include <QHBoxLayout>
 #include <QLabel>
 #include <QMenu>
@@ -33,22 +35,17 @@
 #include <QStyle>
 
 
-DisplayerToolSelect::DisplayerToolSelect(QAction* actionAutodetectLayout, Displayer *displayer, QObject *parent)
-	: DisplayerTool(displayer, parent), mActionAutodetectLayout(actionAutodetectLayout) {
-	connect(mActionAutodetectLayout, SIGNAL(triggered()), this, SLOT(autodetectLayout()));
-
-	MAIN->getConfig()->addSetting(new VarSetting<QString>("selectionsavefile", QDir(Utils::documentsFolder()).absoluteFilePath(_("selection.png"))));
-
-	mActionAutodetectLayout->setVisible(true);
+DisplayerToolSelect::DisplayerToolSelect(Displayer* displayer, QObject* parent)
+	: DisplayerTool(displayer, parent) {
+	displayer->setCursor(Qt::CrossCursor);
 	updateRecognitionModeLabel();
 }
 
 DisplayerToolSelect::~DisplayerToolSelect() {
 	clearSelections();
-	mActionAutodetectLayout->setVisible(false);
 }
 
-void DisplayerToolSelect::mousePressEvent(QMouseEvent *event) {
+void DisplayerToolSelect::mousePressEvent(QMouseEvent* event) {
 	if(event->button() == Qt::LeftButton &&  m_curSel == nullptr) {
 		if((event->modifiers() & Qt::ControlModifier) == 0) {
 			clearSelections();
@@ -60,7 +57,7 @@ void DisplayerToolSelect::mousePressEvent(QMouseEvent *event) {
 	}
 }
 
-void DisplayerToolSelect::mouseMoveEvent(QMouseEvent *event) {
+void DisplayerToolSelect::mouseMoveEvent(QMouseEvent* event) {
 	if(m_curSel) {
 		QPointF p = m_displayer->mapToSceneClamped(event->pos());
 		m_curSel->setPoint(p);
@@ -69,9 +66,9 @@ void DisplayerToolSelect::mouseMoveEvent(QMouseEvent *event) {
 	}
 }
 
-void DisplayerToolSelect::mouseReleaseEvent(QMouseEvent *event) {
+void DisplayerToolSelect::mouseReleaseEvent(QMouseEvent* event) {
 	if(m_curSel) {
-		if(m_curSel->rect().width() < 5. || m_curSel->rect().height() < 5.) {
+		if(m_curSel->rect().width() < 5.0 || m_curSel->rect().height() < 5.0) {
 			delete m_curSel;
 		} else {
 			m_selections.append(m_curSel);
@@ -135,10 +132,8 @@ void DisplayerToolSelect::reorderSelection(int oldNum, int newNum) {
 
 void DisplayerToolSelect::saveSelection(NumberedDisplayerSelection* selection) {
 	QImage img = m_displayer->getImage(selection->rect());
-	QString filename = Utils::makeOutputFilename(MAIN->getConfig()->getSetting<VarSetting<QString>>("selectionsavefile")->getValue());
-	filename = QFileDialog::getSaveFileName(MAIN, _("Save Selection Image"), filename, QString("%1 (*.png)").arg(_("PNG Images")));
+	QString filename = FileDialogs::saveDialog(_("Save Selection Image"), _("selection.png"), "outputdir", QString("%1 (*.png)").arg(_("PNG Images")), true);
 	if(!filename.isEmpty()) {
-		MAIN->getConfig()->getSetting<VarSetting<QString>>("selectionsavefile")->setValue(filename);
 		img.save(filename);
 	}
 }
@@ -150,15 +145,18 @@ void DisplayerToolSelect::updateRecognitionModeLabel() {
 void DisplayerToolSelect::autodetectLayout(bool noDeskew) {
 	clearSelections();
 
-	double avgDeskew = 0.;
+	double avgDeskew = 0.0;
 	int nDeskew = 0;
 	QList<QRectF> rects;
 	QImage img = m_displayer->getImage(m_displayer->getSceneBoundingRect());
 
 	// Perform layout analysis
-	Utils::busyTask([this,&nDeskew,&avgDeskew,&rects,&img] {
+	Utils::busyTask([this, &nDeskew, &avgDeskew, &rects, &img] {
+		QByteArray current = setlocale(LC_ALL, NULL);
+		setlocale(LC_ALL, "C");
 		tesseract::TessBaseAPI tess;
 		tess.InitForAnalysePage();
+		setlocale(LC_ALL, current.constData());
 		tess.SetPageSegMode(tesseract::PSM_AUTO_ONLY);
 		tess.SetImage(img.bits(), img.width(), img.height(), 4, img.bytesPerLine());
 		tesseract::PageIterator* it = tess.AnalyseLayout();
@@ -185,9 +183,10 @@ void DisplayerToolSelect::autodetectLayout(bool noDeskew) {
 
 	// If a somewhat large deskew angle is detected, automatically rotate image and redetect layout,
 	// unless we already attempted to rotate (to prevent endless loops)
-	avgDeskew = qRound(((avgDeskew/nDeskew)/M_PI * 180.) * 10.) / 10.;
-	if(qAbs(avgDeskew) > .1 && !noDeskew) {
-		m_displayer->setAngle(m_displayer->getCurrentAngle() - avgDeskew);
+	avgDeskew = qRound(((avgDeskew / nDeskew) / M_PI * 180.0) * 10.0) / 10.0;
+	if(std::abs(avgDeskew) > 0.1 && !noDeskew) {
+		double newangle = m_displayer->getCurrentAngle() - avgDeskew;
+		m_displayer->setup(nullptr, nullptr, &newangle);
 		autodetectLayout(true);
 	} else {
 		// Merge overlapping rectangles
@@ -211,7 +210,7 @@ void DisplayerToolSelect::autodetectLayout(bool noDeskew) {
 
 ///////////////////////////////////////////////////////////////////////////////
 
-void NumberedDisplayerSelection::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) {
+void NumberedDisplayerSelection::contextMenuEvent(QGraphicsSceneContextMenuEvent* event) {
 	QMenu menu;
 
 	QWidget* orderWidget = new QWidget(&menu);
@@ -258,13 +257,13 @@ void NumberedDisplayerSelection::reorderSelection(int newNumber) {
 	static_cast<DisplayerToolSelect*>(m_tool)->reorderSelection(m_number, newNumber);
 }
 
-void NumberedDisplayerSelection::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) {
+void NumberedDisplayerSelection::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget) {
 	DisplayerSelection::paint(painter, option, widget);
 
 	painter->setRenderHint(QPainter::Antialiasing, false);
 	QRectF r = rect();
-	qreal w = 20. / m_tool->getDisplayer()->getCurrentScale();
-	w = qMin(w, qMin(r.width(), r.height()));
+	qreal w = 20.0 / m_tool->getDisplayer()->getCurrentScale();
+	w = std::min(w, std::min(r.width(), r.height()));
 	QRectF box(r.x(), r.y(), w, w);
 	painter->setBrush(QPalette().highlight());
 	painter->drawRect(box);
