@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * MainWindow.cc
- * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -34,11 +34,13 @@
 #include <sys/prctl.h>
 #endif
 #include <QtSpell.hpp>
+#define USE_STD_NAMESPACE
 #include <tesseract/baseapi.h>
+#undef USE_STD_NAMESPACE
 
 #include "MainWindow.hh"
 #include "Acquirer.hh"
-#include "Config.hh"
+#include "ConfigSettings.hh"
 #include "Displayer.hh"
 #include "DisplayerToolSelect.hh"
 #include "DisplayerToolHOCR.hh"
@@ -46,6 +48,7 @@
 #include "OutputEditorHOCR.hh"
 #include "Recognizer.hh"
 #include "SourceManager.hh"
+#include "TessdataManager.hh"
 #include "Utils.hh"
 #include "ui_AboutDialog.h"
 
@@ -78,14 +81,14 @@ void MainWindow::signalHandler(int signal) {
 	std::raise(signal);
 }
 
-#ifndef __ARMEL__
+#if !(defined(__ARMEL__) || defined(__LCC__) && __LCC__ <= 121)
 static void terminateHandler() {
 	std::set_terminate(nullptr);
 	std::exception_ptr exptr = std::current_exception();
 	if (exptr != 0) {
 		try {
 			std::rethrow_exception(exptr);
-		} catch (std::exception &ex) {
+		} catch (std::exception& ex) {
 			std::cerr << "Terminated due to exception: " << ex.what() << std::endl;
 		} catch (...) {
 			std::cerr << "Terminated due to unknown exception" << std::endl;
@@ -105,13 +108,14 @@ MainWindow::MainWindow(const QStringList& files)
 
 	std::signal(SIGSEGV, signalHandler);
 	std::signal(SIGABRT, signalHandler);
-#ifndef __ARMEL__
+#if !(defined(__ARMEL__) || defined(__LCC__) && __LCC__ <= 121)
 	std::set_terminate(terminateHandler);
 #endif
 
 	QList<QNetworkProxy> listOfProxies = QNetworkProxyFactory::systemProxyForQuery(QNetworkProxyQuery(QUrl(CHECKURL)));
-	if (listOfProxies.size())
+	if (listOfProxies.size()) {
 		QNetworkProxy::setApplicationProxy(listOfProxies[0]);
+	}
 
 	qRegisterMetaType<MainWindow::State>();
 
@@ -124,6 +128,7 @@ MainWindow::MainWindow(const QStringList& files)
 	m_sourceManager = new SourceManager(ui);
 
 	ui.centralwidget->layout()->addWidget(m_displayer);
+	ui.toolBarMain->setLayoutDirection(Qt::LeftToRight);
 
 	m_idleActions.setExclusive(false);
 	m_idleActions.addAction(ui.actionZoomIn);
@@ -141,6 +146,7 @@ MainWindow::MainWindow(const QStringList& files)
 	m_idleWidgets.append(ui.toolButtonRecognize);
 
 	connect(ui.actionRedetectLanguages, SIGNAL(triggered()), m_recognizer, SLOT(updateLanguagesMenu()));
+	connect(ui.actionManageLanguages, SIGNAL(triggered()), this, SLOT(manageLanguages()));
 	connect(ui.actionPreferences, SIGNAL(triggered()), this, SLOT(showConfig()));
 	connect(ui.actionHelp, SIGNAL(triggered()), this, SLOT(showHelp()));
 	connect(ui.actionAbout, SIGNAL(triggered()), this, SLOT(showAbout()));
@@ -149,14 +155,13 @@ MainWindow::MainWindow(const QStringList& files)
 	connect(m_sourceManager, SIGNAL(sourceChanged()), this, SLOT(onSourceChanged()));
 	connect(ui.actionToggleOutputPane, SIGNAL(toggled(bool)), ui.dockWidgetOutput, SLOT(setVisible(bool)));
 	connect(ui.comboBoxOCRMode, SIGNAL(currentIndexChanged(int)), this, SLOT(setOCRMode(int)));
-	connect(m_recognizer, SIGNAL(languageChanged(Config::Lang)), this, SLOT(languageChanged()));
+	connect(m_recognizer, SIGNAL(languageChanged(Config::Lang)), this, SLOT(languageChanged(Config::Lang)));
+	connect(ui.actionAutodetectLayout, SIGNAL(triggered()), m_displayer, SLOT(autodetectOCRAreas()));
 
-
-	m_config->addSetting(new VarSetting<QByteArray>("wingeom"));
-	m_config->addSetting(new VarSetting<QByteArray>("winstate"));
-	m_config->addSetting(new ActionSetting("showcontrols", ui.actionImageControls));
-	m_config->addSetting(new VarSetting<QString>("outputdir", Utils::documentsFolder()));
-	m_config->addSetting(new ComboSetting("outputeditor", ui.comboBoxOCRMode, 0));
+	ADD_SETTING(VarSetting<QByteArray>("wingeom"));
+	ADD_SETTING(VarSetting<QByteArray>("winstate"));
+	ADD_SETTING(ActionSetting("showcontrols", ui.actionImageControls));
+	ADD_SETTING(ComboSetting("outputeditor", ui.comboBoxOCRMode, 0));
 
 	m_recognizer->updateLanguagesMenu();
 
@@ -180,14 +185,14 @@ MainWindow::MainWindow(const QStringList& files)
 
 	pushState(State::Idle, _("Select an image to begin..."));
 
-	restoreGeometry(m_config->getSetting<VarSetting<QByteArray>>("wingeom")->getValue());
-	restoreState(m_config->getSetting<VarSetting<QByteArray>>("winstate")->getValue());
+	restoreGeometry(ConfigSettings::get<VarSetting<QByteArray>>("wingeom")->getValue());
+	restoreState(ConfigSettings::get<VarSetting<QByteArray>>("winstate")->getValue());
 	ui.dockWidgetOutput->setVisible(false);
 
 	ui.actionSources->trigger();
 
 #if ENABLE_VERSIONCHECK
-	if(m_config->getSetting<SwitchSetting>("updatecheck")->getValue()) {
+	if(ConfigSettings::get<SwitchSetting>("updatecheck")->getValue()) {
 		connect(&m_versionCheckThread, SIGNAL(finished()), this, SLOT(checkVersion()));
 		m_versionCheckThread.start();
 	}
@@ -197,6 +202,11 @@ MainWindow::MainWindow(const QStringList& files)
 }
 
 MainWindow::~MainWindow() {
+#if ENABLE_VERSIONCHECK
+	while(m_versionCheckThread.isRunning()) {
+		QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+	}
+#endif
 	delete m_acquirer;
 	delete m_outputEditor;
 	delete m_sourceManager;
@@ -238,8 +248,9 @@ void MainWindow::popState() {
 void MainWindow::setState(State state) {
 	bool isIdle = state == State::Idle;
 	m_idleActions.setEnabled(!isIdle);
-	for(QWidget* widget : m_idleWidgets)
+	for(QWidget* widget : m_idleWidgets) {
 		widget->setEnabled(!isIdle);
+	}
 }
 
 void MainWindow::closeEvent(QCloseEvent* ev) {
@@ -248,20 +259,22 @@ void MainWindow::closeEvent(QCloseEvent* ev) {
 	} else if(!m_outputEditor->clear()) {
 		ev->ignore();
 	} else if(!isMaximized()) {
-		m_config->getSetting<VarSetting<QByteArray>>("wingeom")->setValue(saveGeometry());
-		m_config->getSetting<VarSetting<QByteArray>>("winstate")->setValue(saveState());
+		ConfigSettings::get<VarSetting<QByteArray>>("wingeom")->setValue(saveGeometry());
+		ConfigSettings::get<VarSetting<QByteArray>>("winstate")->setValue(saveState());
 	}
 }
 
 void MainWindow::onSourceChanged() {
-	if(m_stateStack.top().first == State::Normal) {
-		popState();
-	}
 	QList<Source*> sources = m_sourceManager->getSelectedSources();
 	if(m_displayer->setSources(sources)) {
 		setWindowTitle(QString("%1 - %2").arg(sources.size() == 1 ? sources.front()->displayname : _("Multiple sources")).arg(PACKAGE_NAME));
-		pushState(State::Normal, _("Ready"));
+		if(m_stateStack.top().first == State::Idle) {
+			pushState(State::Normal, _("Ready"));
+		}
 	} else {
+		if(m_stateStack.top().first == State::Normal) {
+			popState();
+		}
 		setWindowTitle(PACKAGE_NAME);
 	}
 }
@@ -270,7 +283,7 @@ void MainWindow::showAbout() {
 	QDialog d(this);
 	Ui::AboutDialog aboutDialogUi;
 	aboutDialogUi.setupUi(&d);
-	aboutDialogUi.labelVersion->setText(PACKAGE_VERSION);
+	aboutDialogUi.labelVersion->setText(QString("%1 (%2)").arg(PACKAGE_VERSION, QString(PACKAGE_REVISION).left(6)));;
 	aboutDialogUi.labelTesseractVer->setText(QString("<html><head/><body><p style=\"font-size:small;\">%1 %2</p></body></html>").arg(_("Using tesseract")).arg(TESSERACT_VERSION_STR));
 	d.exec();
 }
@@ -291,6 +304,13 @@ void MainWindow::showHelp(const QString& chapter) {
 	QDesktopServices::openUrl(manualUrl);
 }
 
+void MainWindow::manageLanguages() {
+	TessdataManager manager(MAIN);
+	if(manager.setup()) {
+		manager.exec();
+	}
+}
+
 void MainWindow::showConfig() {
 	m_config->showDialog();
 	m_recognizer->updateLanguagesMenu();
@@ -309,23 +329,23 @@ void MainWindow::setOCRMode(int idx) {
 		delete m_displayerTool;
 		delete m_outputEditor;
 		if(idx == 0) {
-			m_displayerTool = new DisplayerToolSelect(ui.actionAutodetectLayout, m_displayer);
+			m_displayerTool = new DisplayerToolSelect(m_displayer);
 			m_outputEditor = new OutputEditorText();
 		} else { /*if(idx == 1)*/
 			m_displayerTool = new DisplayerToolHOCR(m_displayer);
 			m_outputEditor = new OutputEditorHOCR(static_cast<DisplayerToolHOCR*>(m_displayerTool));
 		}
+		ui.actionAutodetectLayout->setVisible(m_displayerTool->allowAutodetectOCRAreas());
 		m_displayer->setTool(m_displayerTool);
-		connect(m_recognizer, SIGNAL(languageChanged(Config::Lang)), m_outputEditor, SLOT(setLanguage(Config::Lang)));
 		m_outputEditor->setLanguage(m_recognizer->getSelectedLanguage());
 		connect(ui.actionToggleOutputPane, SIGNAL(toggled(bool)), m_outputEditor, SLOT(onVisibilityChanged(bool)));
 		ui.dockWidgetOutput->setWidget(m_outputEditor->getUI());
 	}
 }
 
-void MainWindow::addNotification(const QString& title, const QString& message, const QList<NotificationAction> &actions, MainWindow::Notification* handle) {
+void MainWindow::addNotification(const QString& title, const QString& message, const QList<NotificationAction>& actions, MainWindow::Notification* handle) {
 	QFrame* frame = new QFrame();
-	frame->setFrameStyle(QFrame::StyledPanel|QFrame::Raised);
+	frame->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
 	frame->setStyleSheet("background: #FFD000;");
 	QHBoxLayout* layout = new QHBoxLayout(frame);
 	layout->addWidget(new QLabel(QString("<b>%1</b>").arg(title), frame));
@@ -430,11 +450,14 @@ void MainWindow::progressUpdate() {
 	}
 }
 
-void MainWindow::languageChanged() {
+void MainWindow::languageChanged(const Config::Lang& lang) {
+	if(m_outputEditor) {
+		m_outputEditor->setLanguage(lang);
+	}
 	hideNotification(m_notifierHandle);
 	m_notifierHandle = nullptr;
-	const QString& code = m_recognizer->getSelectedLanguage().code;
-	if(!code.isEmpty() && !QtSpell::checkLanguageInstalled(code) && m_config->getSetting<SwitchSetting>("dictinstall")->getValue()) {
+	const QString& code = lang.code;
+	if(!code.isEmpty() && !QtSpell::checkLanguageInstalled(code) && ConfigSettings::get<SwitchSetting>("dictinstall")->getValue()) {
 		NotificationAction actionDontShowAgain = {_("Don't show again"), m_config, SLOT(disableDictInstall()), true};
 		NotificationAction actionInstall = {_("Install"), this, SLOT(dictionaryAutoinstall()), false};
 #ifdef Q_OS_LINUX
@@ -453,8 +476,7 @@ void MainWindow::languageChanged() {
 }
 
 void MainWindow::dictionaryAutoinstall() {
-	QList<QString> codes = m_config->searchLangCultures(m_recognizer->getSelectedLanguage().code);
-	QString code = codes.isEmpty() ? m_recognizer->getSelectedLanguage().code : codes.front();
+	QString code = m_recognizer->getSelectedLanguage().code;
 	pushState(State::Busy, _("Installing spelling dictionary for '%1'").arg(code));
 #ifdef Q_OS_WIN
 	bool isWindows = true;
@@ -474,7 +496,7 @@ void MainWindow::dictionaryAutoinstall() {
 		req.setArguments(QList<QVariant>() << QVariant::fromValue((quint32)winId()) << QVariant::fromValue(files) << QVariant::fromValue(QString("always")));
 		QDBusMessage reply = QDBusConnection::sessionBus().call(req, QDBus::BlockWithGui, 3600000);
 		if(reply.type() == QDBusMessage::ErrorMessage) {
-			if(QMessageBox::Help == QMessageBox::critical(this, _("Error"), _("Failed to install spelling dictionary: %1").arg(reply.errorMessage()), QMessageBox::Ok|QMessageBox::Help, QMessageBox::Ok)) {
+			if(QMessageBox::Help == QMessageBox::critical(this, _("Error"), _("Failed to install spelling dictionary: %1").arg(reply.errorMessage()), QMessageBox::Ok | QMessageBox::Help, QMessageBox::Ok)) {
 				showHelp("#InstallSpelling");
 			}
 		}
@@ -487,63 +509,65 @@ void MainWindow::dictionaryAutoinstall() {
 		QDir spellingDir(getConfig()->spellingLocation());
 		if(!QDir().mkpath(spellingDir.absolutePath())) {
 			popState();
-			if(QMessageBox::Help == QMessageBox::critical(this, _("Error"), _("Failed to create directory for spelling dictionaries."), QMessageBox::Ok|QMessageBox::Help, QMessageBox::Ok)) {
+			if(QMessageBox::Help == QMessageBox::critical(this, _("Error"), _("Failed to create directory for spelling dictionaries."), QMessageBox::Ok | QMessageBox::Help, QMessageBox::Ok)) {
 				showHelp("#InstallSpelling");
 			}
 			return;
 		}
-		QString urlcode = code;
 		QString messages;
-		QByteArray html = Utils::download(url, messages);
-		if(html.isNull()) {
+		QString html = QString(Utils::download(url, messages));
+		if(html.isEmpty()) {
 			popState();
-			if(QMessageBox::Help == QMessageBox::critical(this, _("Error"), _("Could not read %1: %2.").arg(url).arg(messages), QMessageBox::Ok|QMessageBox::Help, QMessageBox::Ok)) {
-				showHelp("#InstallSpelling");
-			}
-			return;
-		} else if(html.indexOf(QString(">%1<").arg(code)) != -1) {
-			// Ok
-		} else if(html.indexOf(QString(">%1<").arg(code.left(2))) != -1) {
-			urlcode = code.left(2);
-		} else {
-			popState();
-			if(QMessageBox::Help == QMessageBox::critical(this, _("Error"), _("No spelling dictionaries found for '%1'.").arg(code), QMessageBox::Ok|QMessageBox::Help, QMessageBox::Ok)) {
+			if(QMessageBox::Help == QMessageBox::critical(this, _("Error"), _("Could not read %1: %2.").arg(url).arg(messages), QMessageBox::Ok | QMessageBox::Help, QMessageBox::Ok)) {
 				showHelp("#InstallSpelling");
 			}
 			return;
 		}
-		html = Utils::download(url + urlcode + "/", messages);
-		if(html.isNull()) {
-			popState();
-			if(QMessageBox::Help == QMessageBox::critical(this, _("Error"), _("Could not read %1: %2").arg(url + urlcode + "/").arg(messages), QMessageBox::Ok|QMessageBox::Help, QMessageBox::Ok)) {
-				showHelp("#InstallSpelling");
-			}
-			return;
-		}
-		QRegExp pat(QString(">(%1[^<]*\\.(dic|aff))<").arg(code.left(2)));
-		QString htmls = html;
-
-		QString downloaded;
 		int pos = 0;
-		while((pos = htmls.indexOf(pat, pos)) != -1) {
-			pushState(State::Busy, _("Downloading '%1'...").arg(pat.cap(1)));
-			QByteArray data = Utils::download(plainurl + urlcode + "/" + pat.cap(1), messages);
-			if(!data.isNull()) {
-				QFile file(spellingDir.absoluteFilePath(pat.cap(1)));
-				if(file.open(QIODevice::WriteOnly)) {
-					file.write(data);
-					downloaded.append(QString("\n%1").arg(pat.cap(1)));
+		QString langCode = code.left(code.indexOf('_'));
+		QRegExp langPat(QString(">(%1_?[A-Z]*)<").arg(langCode));
+		QRegExp dictPat(QString(">(%1_?[\\w_]*\\.(dic|aff))<").arg(langCode));
+		QStringList downloaded;
+		QStringList failed;
+
+		while((pos = langPat.indexIn(html, pos)) != -1) {
+			QString lang = langPat.cap(1);
+			pos += langPat.matchedLength();
+
+			QString dictHtml = QString(Utils::download(url + lang + "/", messages));
+			if(dictHtml.isEmpty()) {
+				continue;
+			}
+
+			int dictPos = 0;
+			while((dictPos = dictHtml.indexOf(dictPat, dictPos)) != -1) {
+				QString filename = dictPat.cap(1);
+				pushState(State::Busy, _("Downloading '%1'...").arg(filename));
+				QByteArray data = Utils::download(plainurl + lang + "/" + filename, messages);
+				if(!data.isNull()) {
+					QFile file(spellingDir.absoluteFilePath(filename));
+					if(file.open(QIODevice::WriteOnly)) {
+						file.write(data);
+						downloaded.append(filename);
+					} else {
+						failed.append(filename);
+					}
+				} else {
+					failed.append(filename);
 				}
+				popState();
+				dictPos += dictPat.matchedLength();
 			}
-			popState();
-			pos += pat.matchedLength();
 		}
+
 		popState();
-		if(!downloaded.isEmpty()) {
-			QMessageBox::information(this, _("Dictionaries installed"), _("The following dictionary files were installed:%1").arg(downloaded));
+		if(!failed.isEmpty()) {
+			QMessageBox::critical(this, _("Error"), _("The following dictionaries could not be downloaded:\n%1\n\nCheck the connectivity and directory permissions.\nHint: If you don't have write permissions in system folders, you can switch to user paths in the settings dialog.").arg(failed.join("\n")));
+		} else if(!downloaded.isEmpty()) {
 			m_recognizer->updateLanguagesMenu();
+			QMessageBox::information(this, _("Dictionaries installed"), _("The following dictionary files were installed:\n%1").arg(downloaded.join("\n")));
 		} else {
-			if(QMessageBox::Help == QMessageBox::critical(this, _("Error"), _("No spelling dictionaries found for '%1'.").arg(code), QMessageBox::Ok|QMessageBox::Help, QMessageBox::Ok)) {
+			if(QMessageBox::Help == QMessageBox::critical(this, _("Error"), _("No spelling dictionaries found for '%1'.").arg(code), QMessageBox::Ok | QMessageBox::Help, QMessageBox::Ok)) {
 				showHelp("#InstallSpelling");
 			}
 		}
