@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Recognizer.hh
- * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -23,14 +23,17 @@
 #include <QLabel>
 #include <QMessageBox>
 #include <QtSpell.hpp>
+#include <algorithm>
 #include <csignal>
 #include <cstring>
 #include <iostream>
 #include <sstream>
+#define USE_STD_NAMESPACE
 #include <tesseract/baseapi.h>
 #include <tesseract/ocrclass.h>
 #include <tesseract/strngs.h>
 #include <tesseract/genericvector.h>
+#undef USE_STD_NAMESPACE
 #include <QMouseEvent>
 #include <unistd.h>
 #include <setjmp.h>
@@ -40,33 +43,30 @@
 #define pipe(fds) _pipe(fds, 5000, _O_BINARY)
 #endif
 
+#include "ConfigSettings.hh"
 #include "Displayer.hh"
 #include "MainWindow.hh"
 #include "OutputEditor.hh"
 #include "Recognizer.hh"
-#include "TessdataManager.hh"
 #include "Utils.hh"
 
-struct Recognizer::ProgressMonitor : public MainWindow::ProgressMonitor {
+class Recognizer::ProgressMonitor : public MainWindow::ProgressMonitor {
+public:
 	ETEXT_DESC desc;
-	bool canceled = false;
-	int donePages = 0;
-	int nPages;
 
-	ProgressMonitor(int _nPages) {
+	ProgressMonitor(int nPages) : MainWindow::ProgressMonitor(nPages) {
 		desc.progress = 0;
 		desc.cancel = cancelCallback;
 		desc.cancel_this = this;
-		nPages = _nPages;
 	}
-	int getProgress() {
-		return 100 * ((donePages + desc.progress / 100.) / nPages);
-	}
-	void cancel() {
-		canceled = true;
+	int getProgress() const override {
+		QMutexLocker locker(&mMutex);
+		return 100.0 * ((mProgress + desc.progress / 100.0) / mTotal);
 	}
 	static bool cancelCallback(void* instance, int /*words*/) {
-		return reinterpret_cast<ProgressMonitor*>(instance)->canceled;
+		ProgressMonitor* monitor = reinterpret_cast<ProgressMonitor*>(instance);
+		QMutexLocker locker(&monitor->mMutex);
+		return monitor->mCancelled;
 	}
 };
 
@@ -94,22 +94,30 @@ Recognizer::Recognizer(const UI_MainWindow& _ui) :
 	connect(multiplePagesAction, SIGNAL(triggered()), this, SLOT(recognizeMultiplePages()));
 	connect(m_pagesDialogUi.lineEditPageRange, SIGNAL(textChanged(QString)), this, SLOT(clearLineEditPageRangeStyle()));
 
-	MAIN->getConfig()->addSetting(new VarSetting<QString>("language", "eng:en_EN"));
-	MAIN->getConfig()->addSetting(new ComboSetting("ocrregionstrategy", m_pagesDialogUi.comboBoxRecognitionArea, 0));
-	MAIN->getConfig()->addSetting(new SwitchSetting("ocraddsourcefilename", m_pagesDialogUi.checkBoxPrependFilename));
-	MAIN->getConfig()->addSetting(new SwitchSetting("ocraddsourcepage", m_pagesDialogUi.checkBoxPrependPage));
-	MAIN->getConfig()->addSetting(new VarSetting<int>("psm", 6));
+	ADD_SETTING(VarSetting<QString>("language", "eng:en_EN"));
+	ADD_SETTING(ComboSetting("ocrregionstrategy", m_pagesDialogUi.comboBoxRecognitionArea, 0));
+	ADD_SETTING(SwitchSetting("ocraddsourcefilename", m_pagesDialogUi.checkBoxPrependFilename));
+	ADD_SETTING(SwitchSetting("ocraddsourcepage", m_pagesDialogUi.checkBoxPrependPage));
+	ADD_SETTING(VarSetting<int>("psm", 6));
 }
 
 QStringList Recognizer::getAvailableLanguages() const {
-	tesseract::TessBaseAPI tess;
-	initTesseract(tess);
+	tesseract::TessBaseAPI tess = initTesseract();
 	GenericVector<STRING> availLanguages;
 	tess.GetAvailableLanguagesAsVector(&availLanguages);
 	QStringList result;
 	for(int i = 0; i < availLanguages.size(); ++i) {
 		result.append(availLanguages[i].string());
 	}
+	qSort(result.begin(), result.end(), [](const QString & s1, const QString & s2) {
+		bool s1Script = s1.startsWith("script") || s1.left(1) == s1.left(1).toUpper();
+		bool s2Script = s2.startsWith("script") || s2.left(1) == s2.left(1).toUpper();
+		if(s1Script != s2Script) {
+			return !s1Script;
+		} else {
+			return s1 < s2;
+		}
+	});
 	return result;
 }
 
@@ -122,11 +130,11 @@ static void tessCrashHandler(int /*signal*/) {
 	int bytesRead = 0;
 	QString captured;
 	do {
-		if((bytesRead = read(g_pipe[0], buf, sizeof(buf)-1)) > 0) {
+		if((bytesRead = read(g_pipe[0], buf, sizeof(buf) - 1)) > 0) {
 			buf[bytesRead] = 0;
 			captured += buf;
 		}
-	} while(bytesRead == sizeof(buf)-1);
+	} while(bytesRead == sizeof(buf) - 1);
 	tesseract::TessBaseAPI tess;
 	QString errMsg = QString(_("Tesseract crashed with the following message:\n\n"
 	                           "%1\n\n"
@@ -139,9 +147,10 @@ static void tessCrashHandler(int /*signal*/) {
 	longjmp(g_restore_point, SIGSEGV);
 }
 
-bool Recognizer::initTesseract(tesseract::TessBaseAPI& tess, const char* language) const {
-	QByteArray current = setlocale(LC_NUMERIC, NULL);
-	setlocale(LC_NUMERIC, "C");
+tesseract::TessBaseAPI Recognizer::initTesseract(const char* language, bool* ok) const {
+	QByteArray current = setlocale(LC_ALL, NULL);
+	setlocale(LC_ALL, "C");
+	tesseract::TessBaseAPI tess;
 	// unfortunately tesseract creates deliberate segfaults when an error occurs
 	std::signal(SIGSEGV, tessCrashHandler);
 	pipe(g_pipe);
@@ -161,7 +170,10 @@ bool Recognizer::initTesseract(tesseract::TessBaseAPI& tess, const char* languag
 
 	close(g_pipe[0]);
 	close(g_pipe[1]);
-	return ret != -1;
+	if(ok) {
+		*ok = ret != -1;
+	}
+	return tess;
 }
 
 void Recognizer::updateLanguagesMenu() {
@@ -180,7 +192,7 @@ void Recognizer::updateLanguagesMenu() {
 	QAction* activeitem = nullptr;
 	bool haveOsd = false;
 
-	QStringList parts = MAIN->getConfig()->getSetting<VarSetting<QString>>("language")->getValue().split(":");
+	QStringList parts = ConfigSettings::get<VarSetting<QString>>("language")->getValue().split(":");
 	Config::Lang curlang = {parts.empty() ? "eng" : parts[0], parts.size() < 2 ? "" : parts[1], parts.size() < 3 ? "" : parts[2]};
 
 	QList<QString> dicts = QtSpell::Checker::getLanguageList();
@@ -210,7 +222,7 @@ void Recognizer::updateLanguagesMenu() {
 					spelldicts.append(dict);
 				}
 			}
-			qSort(spelldicts);
+			std::sort(spelldicts.begin(), spelldicts.end());
 		}
 		if(!spelldicts.empty()) {
 			QAction* item = new QAction(lang.name, ui.menuLanguages);
@@ -278,13 +290,14 @@ void Recognizer::updateLanguagesMenu() {
 	} else if(activeitem == nullptr) {
 		activeitem = curitem;
 	}
-	if(activeitem)
+	if(activeitem) {
 		activeitem->trigger();
+	}
 
 	// Add PSM items
 	ui.menuLanguages->addSeparator();
 	QMenu* psmMenu = new QMenu();
-	int activePsm = MAIN->getConfig()->getSetting<VarSetting<int>>("psm")->getValue();
+	int activePsm = ConfigSettings::get<VarSetting<int>>("psm")->getValue();
 
 	struct PsmEntry {
 		QString label;
@@ -292,16 +305,17 @@ void Recognizer::updateLanguagesMenu() {
 		bool requireOsd;
 	};
 	QVector<PsmEntry> psmModes = {
-			PsmEntry{_("Automatic page segmentation"), tesseract::PSM_AUTO, false},
-			PsmEntry{_("Page segmentation with orientation and script detection"), tesseract::PSM_AUTO_OSD, true},
-			PsmEntry{_("Assume single column of text"), tesseract::PSM_SINGLE_COLUMN, false},
-			PsmEntry{_("Assume single block of vertically aligned text"), tesseract::PSM_SINGLE_BLOCK_VERT_TEXT, false},
-			PsmEntry{_("Assume a single uniform block of text"), tesseract::PSM_SINGLE_BLOCK, false},
-			PsmEntry{_("Assume a line of text"), tesseract::PSM_SINGLE_LINE, false},
-			PsmEntry{_("Assume a single word"), tesseract::PSM_SINGLE_WORD, false},
-			PsmEntry{_("Assume a single word in a circle"), tesseract::PSM_CIRCLE_WORD, false},
-			PsmEntry{_("Sparse text in no particular order"), tesseract::PSM_SPARSE_TEXT, false},
-			PsmEntry{_("Sparse text with orientation and script detection"), tesseract::PSM_SPARSE_TEXT_OSD, true}};
+		PsmEntry{_("Automatic page segmentation"), tesseract::PSM_AUTO, false},
+		PsmEntry{_("Page segmentation with orientation and script detection"), tesseract::PSM_AUTO_OSD, true},
+		PsmEntry{_("Assume single column of text"), tesseract::PSM_SINGLE_COLUMN, false},
+		PsmEntry{_("Assume single block of vertically aligned text"), tesseract::PSM_SINGLE_BLOCK_VERT_TEXT, false},
+		PsmEntry{_("Assume a single uniform block of text"), tesseract::PSM_SINGLE_BLOCK, false},
+		PsmEntry{_("Assume a line of text"), tesseract::PSM_SINGLE_LINE, false},
+		PsmEntry{_("Assume a single word"), tesseract::PSM_SINGLE_WORD, false},
+		PsmEntry{_("Assume a single word in a circle"), tesseract::PSM_CIRCLE_WORD, false},
+		PsmEntry{_("Sparse text in no particular order"), tesseract::PSM_SPARSE_TEXT, false},
+		PsmEntry{_("Sparse text with orientation and script detection"), tesseract::PSM_SPARSE_TEXT_OSD, true}
+	};
 	for(const auto& entry : psmModes) {
 		QAction* item = psmMenu->addAction(entry.label);
 		item->setData(entry.psmMode);
@@ -318,7 +332,7 @@ void Recognizer::updateLanguagesMenu() {
 
 	// Add installer item
 	ui.menuLanguages->addSeparator();
-	ui.menuLanguages->addAction(_("Manage languages..."), this, SLOT(manageInstalledLanguages()));
+	ui.menuLanguages->addAction(_("Manage languages..."), MAIN, SLOT(manageLanguages()));
 }
 
 void Recognizer::setLanguage() {
@@ -332,7 +346,7 @@ void Recognizer::setLanguage() {
 		}
 		ui.toolButtonRecognize->setText(QString("%1\n%2").arg(m_modeLabel).arg(m_langLabel));
 		m_curLang = lang;
-		MAIN->getConfig()->getSetting<VarSetting<QString>>("language")->setValue(lang.prefix + ":" + lang.code);
+		ConfigSettings::get<VarSetting<QString>>("language")->setValue(lang.prefix + ":" + lang.code);
 		emit languageChanged(m_curLang);
 	}
 }
@@ -352,11 +366,11 @@ void Recognizer::setMultiLanguage() {
 	m_langLabel = langs;
 	ui.toolButtonRecognize->setText(QString("%1\n%2").arg(m_modeLabel).arg(m_langLabel));
 	m_curLang = {langs, "", "Multilingual"};
-	MAIN->getConfig()->getSetting<VarSetting<QString>>("language")->setValue(langs + ":");
+	ConfigSettings::get<VarSetting<QString>>("language")->setValue(langs + ":");
 	emit languageChanged(m_curLang);
 }
 
-void Recognizer::setRecognizeMode(const QString &mode) {
+void Recognizer::setRecognizeMode(const QString& mode) {
 	m_modeLabel = mode;
 	ui.toolButtonRecognize->setText(QString("%1\n%2").arg(m_modeLabel).arg(m_langLabel));
 }
@@ -365,8 +379,8 @@ void Recognizer::clearLineEditPageRangeStyle() {
 	qobject_cast<QLineEdit*>(QObject::sender())->setStyleSheet("");
 }
 
-void Recognizer::psmSelected(QAction *action) {
-	MAIN->getConfig()->getSetting<VarSetting<int>>("psm")->setValue(action->data().toInt());
+void Recognizer::psmSelected(QAction* action) {
+	ConfigSettings::get<VarSetting<int>>("psm")->setValue(action->data().toInt());
 }
 
 QList<int> Recognizer::selectPages(bool& autodetectLayout) {
@@ -380,33 +394,39 @@ QList<int> Recognizer::selectPages(bool& autodetectLayout) {
 
 	m_pagesDialogUi.comboBoxRecognitionArea->setItemText(0, MAIN->getDisplayer()->hasMultipleOCRAreas() ? _("Current selection") : _("Entire page"));
 
+	QRegExp validateRegEx("^[\\d,\\-\\s]+$");
 	QList<int> pages;
-	if(m_pagesDialog->exec() == QDialog::Accepted) {
+	while(m_pagesDialog->exec() == QDialog::Accepted) {
+		pages.clear();
 		QString text = m_pagesDialogUi.lineEditPageRange->text();
-		text.replace(QRegExp("\\s+"), "");
-		for(const QString& block : text.split(',', QString::SkipEmptyParts)) {
-			QStringList ranges = block.split('-', QString::SkipEmptyParts);
-			if(ranges.size() == 1) {
-				int page = ranges[0].toInt();
-				if(page > 0 && page <= nPages) {
-					pages.append(page);
+		if(validateRegEx.indexIn(text) != -1) {
+			text.replace(QRegExp("\\s+"), "");
+			for(const QString& block : text.split(',', QString::SkipEmptyParts)) {
+				QStringList ranges = block.split('-', QString::SkipEmptyParts);
+				if(ranges.size() == 1) {
+					int page = ranges[0].toInt();
+					if(page > 0 && page <= nPages) {
+						pages.append(page);
+					}
+				} else if(ranges.size() == 2) {
+					int start = std::max(1, ranges[0].toInt());
+					int end = std::min(nPages, ranges[1].toInt());
+					for(int page = start; page <= end; ++page) {
+						pages.append(page);
+					}
+				} else {
+					pages.clear();
+					break;
 				}
-			} else if(ranges.size() == 2) {
-				int start = qMax(1, ranges[0].toInt());
-				int end = qMin(nPages, ranges[1].toInt());
-				for(int page = start; page <= end; ++page) {
-					pages.append(page);
-				}
-			} else {
-				pages.clear();
-				break;
 			}
 		}
 		if(pages.empty()) {
 			m_pagesDialogUi.lineEditPageRange->setStyleSheet("background: #FF7777; color: #FFFFFF;");
+		} else {
+			break;
 		}
 	}
-	qSort(pages);
+	std::sort(pages.begin(), pages.end());
 	autodetectLayout = m_pagesDialogUi.comboBoxRecognitionArea->isVisible() ? m_pagesDialogUi.comboBoxRecognitionArea->currentIndex() == 1 : false;
 	return pages;
 }
@@ -434,11 +454,12 @@ void Recognizer::recognizeMultiplePages() {
 	recognize(pages, autodetectLayout);
 }
 
-void Recognizer::recognize(const QList<int> &pages, bool autodetectLayout) {
-	tesseract::TessBaseAPI tess;
-	bool prependFile = pages.size() > 1 && MAIN->getConfig()->getSetting<SwitchSetting>("ocraddsourcefilename")->getValue();
-	bool prependPage = pages.size() > 1 && MAIN->getConfig()->getSetting<SwitchSetting>("ocraddsourcepage")->getValue();
-	if(initTesseract(tess, m_curLang.prefix.toLocal8Bit().constData())) {
+void Recognizer::recognize(const QList<int>& pages, bool autodetectLayout) {
+	bool prependFile = pages.size() > 1 && ConfigSettings::get<SwitchSetting>("ocraddsourcefilename")->getValue();
+	bool prependPage = pages.size() > 1 && ConfigSettings::get<SwitchSetting>("ocraddsourcepage")->getValue();
+	bool ok = false;
+	tesseract::TessBaseAPI tess = initTesseract(m_curLang.prefix.toLocal8Bit().constData(), &ok);
+	if(ok) {
 		QString failed;
 		tess.SetPageSegMode(static_cast<tesseract::PageSegMode>(m_psmCheckGroup->checkedAction()->data().toInt()));
 		OutputEditor::ReadSessionData* readSessionData = MAIN->getOutputEditor()->initRead(tess);
@@ -453,20 +474,22 @@ void Recognizer::recognize(const QList<int> &pages, bool autodetectLayout) {
 				++idx;
 				QMetaObject::invokeMethod(MAIN, "pushState", Qt::QueuedConnection, Q_ARG(MainWindow::State, MainWindow::State::Busy), Q_ARG(QString, _("Recognizing page %1 (%2 of %3)").arg(page).arg(idx).arg(npages)));
 
-				bool success = false;
-				QMetaObject::invokeMethod(this, "setPage", Qt::BlockingQueuedConnection, Q_RETURN_ARG(bool, success), Q_ARG(int, page), Q_ARG(bool, autodetectLayout));
-				if(!success) {
+				PageData pageData;
+				pageData.success = false;
+				QMetaObject::invokeMethod(this, "setPage", Qt::BlockingQueuedConnection, Q_RETURN_ARG(PageData, pageData), Q_ARG(int, page), Q_ARG(bool, autodetectLayout));
+				if(!pageData.success) {
 					failed.append(_("\n- Page %1: failed to render page").arg(page));
 					MAIN->getOutputEditor()->readError(_("\n[Failed to recognize page %1]\n"), readSessionData);
 					continue;
 				}
-				readSessionData->file = MAIN->getDisplayer()->getCurrentImage(readSessionData->page);
-				readSessionData->angle = MAIN->getDisplayer()->getCurrentAngle();
-				readSessionData->resolution = MAIN->getDisplayer()->getCurrentResolution();
+				readSessionData->file = pageData.filename;
+				readSessionData->page = pageData.page;
+				readSessionData->angle = pageData.angle;
+				readSessionData->resolution = pageData.resolution;
 				bool firstChunk = true;
 				bool newFile = readSessionData->file != prevFile;
 				prevFile = readSessionData->file;
-				for(const QImage& image : MAIN->getDisplayer()->getOCRAreas()) {
+				for(const QImage& image : pageData.ocrAreas) {
 					readSessionData->prependPage = prependPage && firstChunk;
 					readSessionData->prependFile = prependFile && (readSessionData->prependPage || newFile);
 					firstChunk = false;
@@ -474,13 +497,13 @@ void Recognizer::recognize(const QList<int> &pages, bool autodetectLayout) {
 					tess.SetImage(image.bits(), image.width(), image.height(), 4, image.bytesPerLine());
 					tess.SetSourceResolution(MAIN->getDisplayer()->getCurrentResolution());
 					tess.Recognize(&monitor.desc);
-					if(!monitor.canceled) {
+					if(!monitor.cancelled()) {
 						MAIN->getOutputEditor()->read(tess, readSessionData);
 					}
 				}
 				QMetaObject::invokeMethod(MAIN, "popState", Qt::QueuedConnection);
-				++monitor.donePages;
-				if(monitor.canceled) {
+				monitor.increaseProgress();
+				if(monitor.cancelled()) {
 					break;
 				}
 			}
@@ -495,8 +518,9 @@ void Recognizer::recognize(const QList<int> &pages, bool autodetectLayout) {
 }
 
 bool Recognizer::recognizeImage(const QImage& image, OutputDestination dest) {
-	tesseract::TessBaseAPI tess;
-	if(!initTesseract(tess, m_curLang.prefix.toLocal8Bit().constData())) {
+	bool ok = false;
+	tesseract::TessBaseAPI tess = initTesseract(m_curLang.prefix.toLocal8Bit().constData(), &ok);
+	if(!ok) {
 		QMessageBox::critical(MAIN, _("Recognition errors occurred"), _("Failed to initialize tesseract"));
 		return false;
 	}
@@ -510,7 +534,7 @@ bool Recognizer::recognizeImage(const QImage& image, OutputDestination dest) {
 		readSessionData->resolution = MAIN->getDisplayer()->getCurrentResolution();
 		Utils::busyTask([&] {
 			tess.Recognize(&monitor.desc);
-			if(!monitor.canceled) {
+			if(!monitor.cancelled()) {
 				MAIN->getOutputEditor()->read(tess, readSessionData);
 			}
 			return true;
@@ -520,7 +544,7 @@ bool Recognizer::recognizeImage(const QImage& image, OutputDestination dest) {
 		QString output;
 		if(Utils::busyTask([&] {
 		tess.Recognize(&monitor.desc);
-			if(!monitor.canceled) {
+			if(!monitor.cancelled()) {
 				char* text = tess.GetUTF8Text();
 				output = QString::fromUtf8(text);
 				delete[] text;
@@ -535,15 +559,19 @@ bool Recognizer::recognizeImage(const QImage& image, OutputDestination dest) {
 	return true;
 }
 
-bool Recognizer::setPage(int page, bool autodetectLayout) {
-	bool success = true;
-	if(page != MAIN->getDisplayer()->getCurrentPage()) {
-		success = MAIN->getDisplayer()->setCurrentPage(page);
+Recognizer::PageData Recognizer::setPage(int page, bool autodetectLayout) {
+	PageData pageData;
+	pageData.success = MAIN->getDisplayer()->setup(&page);
+	if(pageData.success) {
+		if(autodetectLayout) {
+			MAIN->getDisplayer()->autodetectOCRAreas();
+		}
+		pageData.filename = MAIN->getDisplayer()->getCurrentImage(pageData.page);
+		pageData.angle = MAIN->getDisplayer()->getCurrentAngle();
+		pageData.resolution = MAIN->getDisplayer()->getCurrentResolution();
+		pageData.ocrAreas = MAIN->getDisplayer()->getOCRAreas();
 	}
-	if(success && autodetectLayout) {
-		MAIN->getDisplayer()->autodetectOCRAreas();
-	}
-	return success;
+	return pageData;
 }
 
 bool Recognizer::eventFilter(QObject* obj, QEvent* ev) {
@@ -569,10 +597,3 @@ bool Recognizer::eventFilter(QObject* obj, QEvent* ev) {
 	}
 	return QObject::eventFilter(obj, ev);
 }
-
-void Recognizer::manageInstalledLanguages() {
-	TessdataManager manager(MAIN);
-	if(manager.setup()) {
-		manager.exec();
-	}
-}
