@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * SubstitutionsManager.cc
- * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2018 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -20,25 +20,21 @@
 #include <QDesktopServices>
 #include <QDir>
 #include <QFile>
-#include <QFileDialog>
 #include <QFileInfo>
 #include <QList>
 #include <QMessageBox>
 
-#include "SubstitutionsManager.hh"
-#include "MainWindow.hh"
-#include "Config.hh"
 #include "ConfigSettings.hh"
+#include "FileDialogs.hh"
+#include "MainWindow.hh"
 #include "OutputTextEdit.hh"
+#include "SubstitutionsManager.hh"
 #include "Utils.hh"
 
-SubstitutionsManager::SubstitutionsManager(OutputTextEdit* textEdit, QCheckBox* csCheckBox, QWidget* parent)
+SubstitutionsManager::SubstitutionsManager(QWidget* parent)
 	: QDialog(parent) {
 	setWindowTitle(_("Substitutions"));
 
-	m_textEdit = textEdit;
-	m_csCheckBox = csCheckBox;
-
 	QAction* openAction = new QAction(QIcon::fromTheme("document-open"), _("Open"), this);
 	openAction->setToolTip(_("Open"));
 
@@ -76,7 +72,7 @@ SubstitutionsManager::SubstitutionsManager(OutputTextEdit* textEdit, QCheckBox*
 	m_tableWidget->verticalHeader()->setVisible(false);
 	m_tableWidget->setHorizontalHeaderLabels(QStringList() << _("Search for") << _("Replace with"));
 
-	QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Apply|QDialogButtonBox::Close);
+	QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Apply | QDialogButtonBox::Close);
 
 	QVBoxLayout* layout = new QVBoxLayout(this);
 	layout->setMargin(4);
@@ -90,37 +86,33 @@ SubstitutionsManager::SubstitutionsManager(OutputTextEdit* textEdit, QCheckBox*
 	connect(saveAction, SIGNAL(triggered()), this, SLOT(saveList()));
 	connect(clearAction, SIGNAL(triggered()), this, SLOT(clearList()));
 	connect(addAction, SIGNAL(triggered()), this, SLOT(addRow()));
-	connect(buttonBox->button(QDialogButtonBox::Apply), SIGNAL(clicked()), this, SLOT(applySubstitutions()));
+	connect(buttonBox->button(QDialogButtonBox::Apply), SIGNAL(clicked()), this, SLOT(emitApplySubstitutions()));
 	connect(buttonBox->button(QDialogButtonBox::Close), SIGNAL(clicked()), this, SLOT(hide()));
 	connect(m_removeAction, SIGNAL(triggered()), this, SLOT(removeRows()));
-	connect(m_tableWidget->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(onTableSelectionChanged(QItemSelection,QItemSelection)));
+	connect(m_tableWidget->selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this, SLOT(onTableSelectionChanged(QItemSelection, QItemSelection)));
 
-	MAIN->getConfig()->addSetting(new TableSetting("substitutionslist", m_tableWidget));
-	MAIN->getConfig()->addSetting(new VarSetting<QString>("substitutionslistfile", QDir(Utils::documentsFolder()).absoluteFilePath(_("substitution_list.txt"))));
-
-	m_currentFile = MAIN->getConfig()->getSetting<VarSetting<QString>>("substitutionslistfile")->getValue();
+	ADD_SETTING(TableSetting("substitutionslist", m_tableWidget));
 }
 
 SubstitutionsManager::~SubstitutionsManager() {
-	MAIN->getConfig()->removeSetting("replacelist");
-	MAIN->getConfig()->removeSetting("replacelistfile");
+	ConfigSettings::get<TableSetting>("substitutionslist")->serialize();
 }
 
 void SubstitutionsManager::openList() {
 	if(!clearList()) {
 		return;
 	}
-	QString dir = !m_currentFile.isEmpty() ? QFileInfo(m_currentFile).absolutePath() : Utils::documentsFolder();
-	QString filename = QFileDialog::getOpenFileName(this, _("Open Substitutions List"), dir, QString("%1 (*.txt)").arg(_("Substitutions List")));
+	QString dir = !m_currentFile.isEmpty() ? QFileInfo(m_currentFile).absolutePath() : "";
+	QStringList files = FileDialogs::openDialog(_("Open Substitutions List"), dir, "auxdir", QString("%1 (*.txt)").arg(_("Substitutions List")), false, this);
 
-	if(!filename.isEmpty()) {
+	if(!files.isEmpty()) {
+		QString filename = files.front();
 		QFile file(filename);
 		if(!file.open(QIODevice::ReadOnly)) {
 			QMessageBox::critical(this, _("Error Reading File"), _("Unable to read '%1'.").arg(filename));
 			return;
 		}
 		m_currentFile = filename;
-		MAIN->getConfig()->getSetting<VarSetting<QString>>("substitutionslistfile")->setValue(m_currentFile);
 
 		bool errors = false;
 		m_tableWidget->blockSignals(true);
@@ -141,7 +133,6 @@ void SubstitutionsManager::openList() {
 			m_tableWidget->setItem(row, 1, new QTableWidgetItem(fields[1]));
 		}
 		m_tableWidget->blockSignals(false);
-		MAIN->getConfig()->getSetting<TableSetting>("substitutionslist")->serialize();
 		if(errors) {
 			QMessageBox::warning(this, _("Errors Occurred Reading File"), _("Some entries of the substitutions list could not be read."));
 		}
@@ -149,7 +140,7 @@ void SubstitutionsManager::openList() {
 }
 
 bool SubstitutionsManager::saveList() {
-	QString filename = QFileDialog::getSaveFileName(this, _("Save Substitutions List"), m_currentFile, QString("%1 (*.txt)").arg(_("Substitutions List")));
+	QString filename = FileDialogs::saveDialog(_("Save Substitutions List"), m_currentFile, "auxdir", QString("%1 (*.txt)").arg(_("Substitutions List")), false, this);
 	if(filename.isEmpty()) {
 		return false;
 	}
@@ -159,7 +150,6 @@ bool SubstitutionsManager::saveList() {
 		return false;
 	}
 	m_currentFile = filename;
-	MAIN->getConfig()->getSetting<VarSetting<QString>>("substitutionslistfile")->setValue(m_currentFile);
 	for(int row = 0, nRows = m_tableWidget->rowCount(); row < nRows; ++row) {
 		QString line = QString("%1\t%2\n").arg(m_tableWidget->item(row, 0)->text()).arg(m_tableWidget->item(row, 1)->text());
 		file.write(MAIN->getConfig()->useUtf8() ? line.toUtf8() : line.toLocal8Bit());
@@ -178,7 +168,6 @@ bool SubstitutionsManager::clearList() {
 			return false;
 		}
 		m_tableWidget->setRowCount(0);
-		MAIN->getConfig()->getSetting<TableSetting>("substitutionslist")->serialize();
 	}
 	return true;
 }
@@ -197,37 +186,16 @@ void SubstitutionsManager::removeRows() {
 		m_tableWidget->removeRow(index.row());
 	}
 	m_tableWidget->blockSignals(false);
-	MAIN->getConfig()->getSetting<TableSetting>("substitutionslist")->serialize();
 }
 
 void SubstitutionsManager::onTableSelectionChanged(const QItemSelection& selected, const QItemSelection& /*deselected*/) {
 	m_removeAction->setEnabled(!selected.isEmpty());
 }
 
-void SubstitutionsManager::applySubstitutions() {
-	MAIN->pushState(MainWindow::State::Busy, _("Applying substitutions..."));
-	QTextCursor cursor =  m_textEdit->regionBounds();
-	int end = cursor.position();
-	cursor.setPosition(cursor.anchor());
-	QTextDocument::FindFlags flags = 0;
-	if(m_csCheckBox->isChecked()) {
-		flags = QTextDocument::FindCaseSensitively;
-	}
-	int start = cursor.position();
+void SubstitutionsManager::emitApplySubstitutions() {
+	QMap<QString, QString> substitutions;
 	for(int row = 0, nRows = m_tableWidget->rowCount(); row < nRows; ++row) {
-		QString search = m_tableWidget->item(row, 0)->text();
-		QString replace = m_tableWidget->item(row, 1)->text();
-		int diff = replace.length() - search.length();
-		cursor.setPosition(start);
-		while(true) {
-			cursor = m_textEdit->document()->find(search, cursor, flags);
-			if(cursor.isNull() || qMax(cursor.anchor(), cursor.position()) > end) {
-				break;
-			}
-			cursor.insertText(replace);
-			end += diff;
-		}
-		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+		substitutions.insert(m_tableWidget->item(row, 0)->text(), m_tableWidget->item(row, 1)->text());
 	}
-	MAIN->popState();
+	emit applySubstitutions(substitutions);
 }
